# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
from . import databricks_pb2 as databricks__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rservice.proto\x12\x06mlflow\x1a\x15scalapb/scalapb.proto\x1a\x10\x64\x61tabricks.proto\"H\n\x06Metric\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\x12\x0f\n\x04step\x18\x04 \x01(\x03:\x01\x30\"#\n\x05Param\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"f\n\x03Run\x12\x1d\n\x04info\x18\x01 \x01(\x0b\x32\x0f.mlflow.RunInfo\x12\x1d\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x0f.mlflow.RunData\x12!\n\x06inputs\x18\x03 \x01(\x0b\x32\x11.mlflow.RunInputs\"g\n\x07RunData\x12\x1f\n\x07metrics\x18\x01 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x1d\n\x06params\x18\x02 \x03(\x0b\x32\r.mlflow.Param\x12\x1c\n\x04tags\x18\x03 \x03(\x0b\x32\x0e.mlflow.RunTag\"9\n\tRunInputs\x12,\n\x0e\x64\x61taset_inputs\x18\x01 \x03(\x0b\x32\x14.mlflow.DatasetInput\"$\n\x06RunTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"+\n\rExperimentTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xdd\x01\n\x07RunInfo\x12\x0e\n\x06run_id\x18\x0f \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x10\n\x08run_name\x18\x03 \x01(\t\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x0f\n\x07user_id\x18\x06 \x01(\t\x12!\n\x06status\x18\x07 \x01(\x0e\x32\x11.mlflow.RunStatus\x12\x12\n\nstart_time\x18\x08 \x01(\x03\x12\x10\n\x08\x65nd_time\x18\t \x01(\x03\x12\x14\n\x0c\x61rtifact_uri\x18\r \x01(\t\x12\x17\n\x0flifecycle_stage\x18\x0e \x01(\t\"\xbb\x01\n\nExperiment\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x19\n\x11\x61rtifact_location\x18\x03 \x01(\t\x12\x17\n\x0flifecycle_stage\x18\x04 \x01(\t\x12\x18\n\x10last_update_time\x18\x05 \x01(\x03\x12\x15\n\rcreation_time\x18\x06 \x01(\x03\x12#\n\x04tags\x18\x07 \x03(\x0b\x32\x15.mlflow.ExperimentTag\"V\n\x0c\x44\x61tasetInput\x12\x1e\n\x04tags\x18\x01 \x03(\x0b\x32\x10.mlflow.InputTag\x12&\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32\x0f.mlflow.DatasetB\x04\xf8\x86\x19\x01\"2\n\x08InputTag\x12\x11\n\x03key\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\"\x85\x01\n\x07\x44\x61taset\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06\x64igest\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0bsource_type\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06source\x18\x04 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0e\n\x06schema\x18\x05 \x01(\t\x12\x0f\n\x07profile\x18\x06 \x01(\t\"\xb6\x01\n\x10\x43reateExperiment\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x11\x61rtifact_location\x18\x02 \x01(\t\x12#\n\x04tags\x18\x03 \x03(\x0b\x32\x15.mlflow.ExperimentTag\x1a!\n\x08Response\x12\x15\n\rexperiment_id\x18\x01 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xfe\x01\n\x11SearchExperiments\x12\x13\n\x0bmax_results\x18\x01 \x01(\x03\x12\x12\n\npage_token\x18\x02 \x01(\t\x12\x0e\n\x06\x66ilter\x18\x03 \x01(\t\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12#\n\tview_type\x18\x05 \x01(\x0e\x32\x10.mlflow.ViewType\x1aL\n\x08Response\x12\'\n\x0b\x65xperiments\x18\x01 \x03(\x0b\x32\x12.mlflow.Experiment\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8d\x01\n\rGetExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x32\n\x08Response\x12&\n\nexperiment\x18\x01 \x01(\x0b\x32\x12.mlflow.Experiment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"h\n\x10\x44\x65leteExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"i\n\x11RestoreExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"z\n\x10UpdateExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x10\n\x08new_name\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xca\x01\n\tCreateRun\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x10\n\x08run_name\x18\x03 \x01(\t\x12\x12\n\nstart_time\x18\x07 \x01(\x03\x12\x1c\n\x04tags\x18\t \x03(\x0b\x32\x0e.mlflow.RunTag\x1a$\n\x08Response\x12\x18\n\x03run\x18\x01 \x01(\x0b\x32\x0b.mlflow.Run:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd0\x01\n\tUpdateRun\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12!\n\x06status\x18\x02 \x01(\x0e\x32\x11.mlflow.RunStatus\x12\x10\n\x08\x65nd_time\x18\x03 \x01(\x03\x12\x10\n\x08run_name\x18\x05 \x01(\t\x1a-\n\x08Response\x12!\n\x08run_info\x18\x01 \x01(\x0b\x32\x0f.mlflow.RunInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"Z\n\tDeleteRun\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"[\n\nRestoreRun\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb8\x01\n\tLogMetric\x12\x0e\n\x06run_id\x18\x06 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\x01\x42\x04\xf8\x86\x19\x01\x12\x17\n\ttimestamp\x18\x04 \x01(\x03\x42\x04\xf8\x86\x19\x01\x12\x0f\n\x04step\x18\x05 \x01(\x03:\x01\x30\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8d\x01\n\x08LogParam\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x90\x01\n\x10SetExperimentTag\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8b\x01\n\x06SetTag\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"m\n\tDeleteTag\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"}\n\x06GetRun\x12\x0e\n\x06run_id\x18\x02 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x1a$\n\x08Response\x12\x18\n\x03run\x18\x01 \x01(\x0b\x32\x0b.mlflow.Run:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x98\x02\n\nSearchRuns\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12\x34\n\rrun_view_type\x18\x03 \x01(\x0e\x32\x10.mlflow.ViewType:\x0b\x41\x43TIVE_ONLY\x12\x19\n\x0bmax_results\x18\x05 \x01(\x05:\x04\x31\x30\x30\x30\x12\x10\n\x08order_by\x18\x06 \x03(\t\x12\x12\n\npage_token\x18\x07 \x01(\t\x1a>\n\x08Response\x12\x19\n\x04runs\x18\x01 \x03(\x0b\x32\x0b.mlflow.Run\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd8\x01\n\rListArtifacts\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x12\n\npage_token\x18\x04 \x01(\t\x1aV\n\x08Response\x12\x10\n\x08root_uri\x18\x01 \x01(\t\x12\x1f\n\x05\x66iles\x18\x02 \x03(\x0b\x32\x10.mlflow.FileInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\";\n\x08\x46ileInfo\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06is_dir\x18\x02 \x01(\x08\x12\x11\n\tfile_size\x18\x03 \x01(\x03\"\xea\x01\n\x10GetMetricHistory\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x18\n\nmetric_key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\npage_token\x18\x04 \x01(\t\x12\x13\n\x0bmax_results\x18\x05 \x01(\x05\x1a\x44\n\x08Response\x12\x1f\n\x07metrics\x18\x01 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"a\n\x0fMetricWithRunId\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\x12\x0f\n\x04step\x18\x04 \x01(\x03:\x01\x30\x12\x0e\n\x06run_id\x18\x05 \x01(\t\"\x9b\x02\n\x1cGetMetricHistoryBulkInterval\x12\x0f\n\x07run_ids\x18\x01 \x03(\t\x12\x18\n\nmetric_key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\nstart_step\x18\x03 \x01(\x05\x12\x10\n\x08\x65nd_step\x18\x04 \x01(\x05\x12\x13\n\x0bmax_results\x18\x05 \x01(\x05\x1a\x34\n\x08Response\x12(\n\x07metrics\x18\x01 \x03(\x0b\x32\x17.mlflow.MetricWithRunId:_\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\xe2?1\n/com.databricks.mlflow.api.MlflowTrackingMessage\"\xb1\x01\n\x08LogBatch\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x1f\n\x07metrics\x18\x02 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x1d\n\x06params\x18\x03 \x03(\x0b\x32\r.mlflow.Param\x12\x1c\n\x04tags\x18\x04 \x03(\x0b\x32\x0e.mlflow.RunTag\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"g\n\x08LogModel\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x12\n\nmodel_json\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb6\x01\n\tLogInputs\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12&\n\x08\x64\x61tasets\x18\x02 \x03(\x0b\x32\x14.mlflow.DatasetInput\x1a\n\n\x08Response:_\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\xe2?1\n/com.databricks.mlflow.api.MlflowTrackingMessage\"\x95\x01\n\x13GetExperimentByName\x12\x1d\n\x0f\x65xperiment_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x32\n\x08Response\x12&\n\nexperiment\x18\x01 \x01(\x0b\x32\x12.mlflow.Experiment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]*6\n\x08ViewType\x12\x0f\n\x0b\x41\x43TIVE_ONLY\x10\x01\x12\x10\n\x0c\x44\x45LETED_ONLY\x10\x02\x12\x07\n\x03\x41LL\x10\x03*I\n\nSourceType\x12\x0c\n\x08NOTEBOOK\x10\x01\x12\x07\n\x03JOB\x10\x02\x12\x0b\n\x07PROJECT\x10\x03\x12\t\n\x05LOCAL\x10\x04\x12\x0c\n\x07UNKNOWN\x10\xe8\x07*M\n\tRunStatus\x12\x0b\n\x07RUNNING\x10\x01\x12\r\n\tSCHEDULED\x10\x02\x12\x0c\n\x08\x46INISHED\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\n\n\x06KILLED\x10\x05\x32\xb6\x19\n\rMlflowService\x12\xa6\x01\n\x13getExperimentByName\x12\x1b.mlflow.GetExperimentByName\x1a$.mlflow.GetExperimentByName.Response\"L\xf2\x86\x19H\n,\n\x03GET\x12\x1f/mlflow/experiments/get-by-name\x1a\x04\x08\x02\x10\x00\x10\x01*\x16Get Experiment By Name\x12\x94\x01\n\x10\x63reateExperiment\x12\x18.mlflow.CreateExperiment\x1a!.mlflow.CreateExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/create\x1a\x04\x08\x02\x10\x00\x10\x01*\x11\x43reate Experiment\x12\xc1\x01\n\x11searchExperiments\x12\x19.mlflow.SearchExperiments\x1a\".mlflow.SearchExperiments.Response\"m\xf2\x86\x19i\n(\n\x04POST\x12\x1a/mlflow/experiments/search\x1a\x04\x08\x02\x10\x00\n\'\n\x03GET\x12\x1a/mlflow/experiments/search\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Search Experiments\x12\x88\x01\n\rgetExperiment\x12\x15.mlflow.GetExperiment\x1a\x1e.mlflow.GetExperiment.Response\"@\xf2\x86\x19\x38\n$\n\x03GET\x12\x17/mlflow/experiments/get\x1a\x04\x08\x02\x10\x00\x10\x01*\x0eGet Experiment\xba\x8c\x19\x00\x12\x94\x01\n\x10\x64\x65leteExperiment\x12\x18.mlflow.DeleteExperiment\x1a!.mlflow.DeleteExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\x11\x44\x65lete Experiment\x12\x99\x01\n\x11restoreExperiment\x12\x19.mlflow.RestoreExperiment\x1a\".mlflow.RestoreExperiment.Response\"E\xf2\x86\x19\x41\n)\n\x04POST\x12\x1b/mlflow/experiments/restore\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Restore Experiment\x12\x94\x01\n\x10updateExperiment\x12\x18.mlflow.UpdateExperiment\x1a!.mlflow.UpdateExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/update\x1a\x04\x08\x02\x10\x00\x10\x01*\x11Update Experiment\x12q\n\tcreateRun\x12\x11.mlflow.CreateRun\x1a\x1a.mlflow.CreateRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/create\x1a\x04\x08\x02\x10\x00\x10\x01*\nCreate Run\x12q\n\tupdateRun\x12\x11.mlflow.UpdateRun\x1a\x1a.mlflow.UpdateRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/update\x1a\x04\x08\x02\x10\x00\x10\x01*\nUpdate Run\x12q\n\tdeleteRun\x12\x11.mlflow.DeleteRun\x1a\x1a.mlflow.DeleteRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\nDelete Run\x12v\n\nrestoreRun\x12\x12.mlflow.RestoreRun\x1a\x1b.mlflow.RestoreRun.Response\"7\xf2\x86\x19\x33\n\"\n\x04POST\x12\x14/mlflow/runs/restore\x1a\x04\x08\x02\x10\x00\x10\x01*\x0bRestore Run\x12u\n\tlogMetric\x12\x11.mlflow.LogMetric\x1a\x1a.mlflow.LogMetric.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/log-metric\x1a\x04\x08\x02\x10\x00\x10\x01*\nLog Metric\x12t\n\x08logParam\x12\x10.mlflow.LogParam\x1a\x19.mlflow.LogParam.Response\";\xf2\x86\x19\x37\n(\n\x04POST\x12\x1a/mlflow/runs/log-parameter\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Param\x12\xa1\x01\n\x10setExperimentTag\x12\x18.mlflow.SetExperimentTag\x1a!.mlflow.SetExperimentTag.Response\"P\xf2\x86\x19L\n4\n\x04POST\x12&/mlflow/experiments/set-experiment-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Set Experiment Tag\x12\x66\n\x06setTag\x12\x0e.mlflow.SetTag\x1a\x17.mlflow.SetTag.Response\"3\xf2\x86\x19/\n\"\n\x04POST\x12\x14/mlflow/runs/set-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\x07Set Tag\x12u\n\tdeleteTag\x12\x11.mlflow.DeleteTag\x1a\x1a.mlflow.DeleteTag.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/delete-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\nDelete Tag\x12\x65\n\x06getRun\x12\x0e.mlflow.GetRun\x1a\x17.mlflow.GetRun.Response\"2\xf2\x86\x19*\n\x1d\n\x03GET\x12\x10/mlflow/runs/get\x1a\x04\x08\x02\x10\x00\x10\x01*\x07Get Run\xba\x8c\x19\x00\x12u\n\nsearchRuns\x12\x12.mlflow.SearchRuns\x1a\x1b.mlflow.SearchRuns.Response\"6\xf2\x86\x19\x32\n!\n\x04POST\x12\x13/mlflow/runs/search\x1a\x04\x08\x02\x10\x00\x10\x01*\x0bSearch Runs\x12\x83\x01\n\rlistArtifacts\x12\x15.mlflow.ListArtifacts\x1a\x1e.mlflow.ListArtifacts.Response\";\xf2\x86\x19\x37\n#\n\x03GET\x12\x16/mlflow/artifacts/list\x1a\x04\x08\x02\x10\x00\x10\x01*\x0eList Artifacts\x12\x95\x01\n\x10getMetricHistory\x12\x18.mlflow.GetMetricHistory\x1a!.mlflow.GetMetricHistory.Response\"D\xf2\x86\x19@\n(\n\x03GET\x12\x1b/mlflow/metrics/get-history\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Get Metric History\x12\xb3\x01\n\x1cgetMetricHistoryBulkInterval\x12$.mlflow.GetMetricHistoryBulkInterval\x1a-.mlflow.GetMetricHistoryBulkInterval.Response\">\xf2\x86\x19:\n6\n\x03GET\x12)/mlflow/metrics/get-history-bulk-interval\x1a\x04\x08\x02\x10\x0b\x10\x03\x12p\n\x08logBatch\x12\x10.mlflow.LogBatch\x1a\x19.mlflow.LogBatch.Response\"7\xf2\x86\x19\x33\n$\n\x04POST\x12\x16/mlflow/runs/log-batch\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Batch\x12p\n\x08logModel\x12\x10.mlflow.LogModel\x1a\x19.mlflow.LogModel.Response\"7\xf2\x86\x19\x33\n$\n\x04POST\x12\x16/mlflow/runs/log-model\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Model\x12u\n\tlogInputs\x12\x11.mlflow.LogInputs\x1a\x1a.mlflow.LogInputs.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/log-inputs\x1a\x04\x08\x02\x10\x00\x10\x01*\nLog InputsB\x1e\n\x14org.mlflow.api.proto\x90\x01\x01\xe2?\x02\x10\x01')

_VIEWTYPE = DESCRIPTOR.enum_types_by_name['ViewType']
ViewType = enum_type_wrapper.EnumTypeWrapper(_VIEWTYPE)
_SOURCETYPE = DESCRIPTOR.enum_types_by_name['SourceType']
SourceType = enum_type_wrapper.EnumTypeWrapper(_SOURCETYPE)
_RUNSTATUS = DESCRIPTOR.enum_types_by_name['RunStatus']
RunStatus = enum_type_wrapper.EnumTypeWrapper(_RUNSTATUS)
ACTIVE_ONLY = 1
DELETED_ONLY = 2
ALL = 3
NOTEBOOK = 1
JOB = 2
PROJECT = 3
LOCAL = 4
UNKNOWN = 1000
RUNNING = 1
SCHEDULED = 2
FINISHED = 3
FAILED = 4
KILLED = 5


_METRIC = DESCRIPTOR.message_types_by_name['Metric']
_PARAM = DESCRIPTOR.message_types_by_name['Param']
_RUN = DESCRIPTOR.message_types_by_name['Run']
_RUNDATA = DESCRIPTOR.message_types_by_name['RunData']
_RUNINPUTS = DESCRIPTOR.message_types_by_name['RunInputs']
_RUNTAG = DESCRIPTOR.message_types_by_name['RunTag']
_EXPERIMENTTAG = DESCRIPTOR.message_types_by_name['ExperimentTag']
_RUNINFO = DESCRIPTOR.message_types_by_name['RunInfo']
_EXPERIMENT = DESCRIPTOR.message_types_by_name['Experiment']
_DATASETINPUT = DESCRIPTOR.message_types_by_name['DatasetInput']
_INPUTTAG = DESCRIPTOR.message_types_by_name['InputTag']
_DATASET = DESCRIPTOR.message_types_by_name['Dataset']
_CREATEEXPERIMENT = DESCRIPTOR.message_types_by_name['CreateExperiment']
_CREATEEXPERIMENT_RESPONSE = _CREATEEXPERIMENT.nested_types_by_name['Response']
_SEARCHEXPERIMENTS = DESCRIPTOR.message_types_by_name['SearchExperiments']
_SEARCHEXPERIMENTS_RESPONSE = _SEARCHEXPERIMENTS.nested_types_by_name['Response']
_GETEXPERIMENT = DESCRIPTOR.message_types_by_name['GetExperiment']
_GETEXPERIMENT_RESPONSE = _GETEXPERIMENT.nested_types_by_name['Response']
_DELETEEXPERIMENT = DESCRIPTOR.message_types_by_name['DeleteExperiment']
_DELETEEXPERIMENT_RESPONSE = _DELETEEXPERIMENT.nested_types_by_name['Response']
_RESTOREEXPERIMENT = DESCRIPTOR.message_types_by_name['RestoreExperiment']
_RESTOREEXPERIMENT_RESPONSE = _RESTOREEXPERIMENT.nested_types_by_name['Response']
_UPDATEEXPERIMENT = DESCRIPTOR.message_types_by_name['UpdateExperiment']
_UPDATEEXPERIMENT_RESPONSE = _UPDATEEXPERIMENT.nested_types_by_name['Response']
_CREATERUN = DESCRIPTOR.message_types_by_name['CreateRun']
_CREATERUN_RESPONSE = _CREATERUN.nested_types_by_name['Response']
_UPDATERUN = DESCRIPTOR.message_types_by_name['UpdateRun']
_UPDATERUN_RESPONSE = _UPDATERUN.nested_types_by_name['Response']
_DELETERUN = DESCRIPTOR.message_types_by_name['DeleteRun']
_DELETERUN_RESPONSE = _DELETERUN.nested_types_by_name['Response']
_RESTORERUN = DESCRIPTOR.message_types_by_name['RestoreRun']
_RESTORERUN_RESPONSE = _RESTORERUN.nested_types_by_name['Response']
_LOGMETRIC = DESCRIPTOR.message_types_by_name['LogMetric']
_LOGMETRIC_RESPONSE = _LOGMETRIC.nested_types_by_name['Response']
_LOGPARAM = DESCRIPTOR.message_types_by_name['LogParam']
_LOGPARAM_RESPONSE = _LOGPARAM.nested_types_by_name['Response']
_SETEXPERIMENTTAG = DESCRIPTOR.message_types_by_name['SetExperimentTag']
_SETEXPERIMENTTAG_RESPONSE = _SETEXPERIMENTTAG.nested_types_by_name['Response']
_SETTAG = DESCRIPTOR.message_types_by_name['SetTag']
_SETTAG_RESPONSE = _SETTAG.nested_types_by_name['Response']
_DELETETAG = DESCRIPTOR.message_types_by_name['DeleteTag']
_DELETETAG_RESPONSE = _DELETETAG.nested_types_by_name['Response']
_GETRUN = DESCRIPTOR.message_types_by_name['GetRun']
_GETRUN_RESPONSE = _GETRUN.nested_types_by_name['Response']
_SEARCHRUNS = DESCRIPTOR.message_types_by_name['SearchRuns']
_SEARCHRUNS_RESPONSE = _SEARCHRUNS.nested_types_by_name['Response']
_LISTARTIFACTS = DESCRIPTOR.message_types_by_name['ListArtifacts']
_LISTARTIFACTS_RESPONSE = _LISTARTIFACTS.nested_types_by_name['Response']
_FILEINFO = DESCRIPTOR.message_types_by_name['FileInfo']
_GETMETRICHISTORY = DESCRIPTOR.message_types_by_name['GetMetricHistory']
_GETMETRICHISTORY_RESPONSE = _GETMETRICHISTORY.nested_types_by_name['Response']
_METRICWITHRUNID = DESCRIPTOR.message_types_by_name['MetricWithRunId']
_GETMETRICHISTORYBULKINTERVAL = DESCRIPTOR.message_types_by_name['GetMetricHistoryBulkInterval']
_GETMETRICHISTORYBULKINTERVAL_RESPONSE = _GETMETRICHISTORYBULKINTERVAL.nested_types_by_name['Response']
_LOGBATCH = DESCRIPTOR.message_types_by_name['LogBatch']
_LOGBATCH_RESPONSE = _LOGBATCH.nested_types_by_name['Response']
_LOGMODEL = DESCRIPTOR.message_types_by_name['LogModel']
_LOGMODEL_RESPONSE = _LOGMODEL.nested_types_by_name['Response']
_LOGINPUTS = DESCRIPTOR.message_types_by_name['LogInputs']
_LOGINPUTS_RESPONSE = _LOGINPUTS.nested_types_by_name['Response']
_GETEXPERIMENTBYNAME = DESCRIPTOR.message_types_by_name['GetExperimentByName']
_GETEXPERIMENTBYNAME_RESPONSE = _GETEXPERIMENTBYNAME.nested_types_by_name['Response']
Metric = _reflection.GeneratedProtocolMessageType('Metric', (_message.Message,), {
  'DESCRIPTOR' : _METRIC,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.Metric)
  })
_sym_db.RegisterMessage(Metric)

Param = _reflection.GeneratedProtocolMessageType('Param', (_message.Message,), {
  'DESCRIPTOR' : _PARAM,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.Param)
  })
_sym_db.RegisterMessage(Param)

Run = _reflection.GeneratedProtocolMessageType('Run', (_message.Message,), {
  'DESCRIPTOR' : _RUN,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.Run)
  })
_sym_db.RegisterMessage(Run)

RunData = _reflection.GeneratedProtocolMessageType('RunData', (_message.Message,), {
  'DESCRIPTOR' : _RUNDATA,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RunData)
  })
_sym_db.RegisterMessage(RunData)

RunInputs = _reflection.GeneratedProtocolMessageType('RunInputs', (_message.Message,), {
  'DESCRIPTOR' : _RUNINPUTS,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RunInputs)
  })
_sym_db.RegisterMessage(RunInputs)

RunTag = _reflection.GeneratedProtocolMessageType('RunTag', (_message.Message,), {
  'DESCRIPTOR' : _RUNTAG,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RunTag)
  })
_sym_db.RegisterMessage(RunTag)

ExperimentTag = _reflection.GeneratedProtocolMessageType('ExperimentTag', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTTAG,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.ExperimentTag)
  })
_sym_db.RegisterMessage(ExperimentTag)

RunInfo = _reflection.GeneratedProtocolMessageType('RunInfo', (_message.Message,), {
  'DESCRIPTOR' : _RUNINFO,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RunInfo)
  })
_sym_db.RegisterMessage(RunInfo)

Experiment = _reflection.GeneratedProtocolMessageType('Experiment', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENT,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.Experiment)
  })
_sym_db.RegisterMessage(Experiment)

DatasetInput = _reflection.GeneratedProtocolMessageType('DatasetInput', (_message.Message,), {
  'DESCRIPTOR' : _DATASETINPUT,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.DatasetInput)
  })
_sym_db.RegisterMessage(DatasetInput)

InputTag = _reflection.GeneratedProtocolMessageType('InputTag', (_message.Message,), {
  'DESCRIPTOR' : _INPUTTAG,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.InputTag)
  })
_sym_db.RegisterMessage(InputTag)

Dataset = _reflection.GeneratedProtocolMessageType('Dataset', (_message.Message,), {
  'DESCRIPTOR' : _DATASET,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.Dataset)
  })
_sym_db.RegisterMessage(Dataset)

CreateExperiment = _reflection.GeneratedProtocolMessageType('CreateExperiment', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _CREATEEXPERIMENT_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateExperiment.Response)
    })
  ,
  'DESCRIPTOR' : _CREATEEXPERIMENT,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.CreateExperiment)
  })
_sym_db.RegisterMessage(CreateExperiment)
_sym_db.RegisterMessage(CreateExperiment.Response)

SearchExperiments = _reflection.GeneratedProtocolMessageType('SearchExperiments', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _SEARCHEXPERIMENTS_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchExperiments.Response)
    })
  ,
  'DESCRIPTOR' : _SEARCHEXPERIMENTS,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.SearchExperiments)
  })
_sym_db.RegisterMessage(SearchExperiments)
_sym_db.RegisterMessage(SearchExperiments.Response)

GetExperiment = _reflection.GeneratedProtocolMessageType('GetExperiment', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETEXPERIMENT_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetExperiment.Response)
    })
  ,
  'DESCRIPTOR' : _GETEXPERIMENT,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetExperiment)
  })
_sym_db.RegisterMessage(GetExperiment)
_sym_db.RegisterMessage(GetExperiment.Response)

DeleteExperiment = _reflection.GeneratedProtocolMessageType('DeleteExperiment', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _DELETEEXPERIMENT_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteExperiment.Response)
    })
  ,
  'DESCRIPTOR' : _DELETEEXPERIMENT,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.DeleteExperiment)
  })
_sym_db.RegisterMessage(DeleteExperiment)
_sym_db.RegisterMessage(DeleteExperiment.Response)

RestoreExperiment = _reflection.GeneratedProtocolMessageType('RestoreExperiment', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _RESTOREEXPERIMENT_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RestoreExperiment.Response)
    })
  ,
  'DESCRIPTOR' : _RESTOREEXPERIMENT,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RestoreExperiment)
  })
_sym_db.RegisterMessage(RestoreExperiment)
_sym_db.RegisterMessage(RestoreExperiment.Response)

UpdateExperiment = _reflection.GeneratedProtocolMessageType('UpdateExperiment', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _UPDATEEXPERIMENT_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateExperiment.Response)
    })
  ,
  'DESCRIPTOR' : _UPDATEEXPERIMENT,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.UpdateExperiment)
  })
_sym_db.RegisterMessage(UpdateExperiment)
_sym_db.RegisterMessage(UpdateExperiment.Response)

CreateRun = _reflection.GeneratedProtocolMessageType('CreateRun', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _CREATERUN_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateRun.Response)
    })
  ,
  'DESCRIPTOR' : _CREATERUN,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.CreateRun)
  })
_sym_db.RegisterMessage(CreateRun)
_sym_db.RegisterMessage(CreateRun.Response)

UpdateRun = _reflection.GeneratedProtocolMessageType('UpdateRun', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _UPDATERUN_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateRun.Response)
    })
  ,
  'DESCRIPTOR' : _UPDATERUN,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.UpdateRun)
  })
_sym_db.RegisterMessage(UpdateRun)
_sym_db.RegisterMessage(UpdateRun.Response)

DeleteRun = _reflection.GeneratedProtocolMessageType('DeleteRun', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _DELETERUN_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteRun.Response)
    })
  ,
  'DESCRIPTOR' : _DELETERUN,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.DeleteRun)
  })
_sym_db.RegisterMessage(DeleteRun)
_sym_db.RegisterMessage(DeleteRun.Response)

RestoreRun = _reflection.GeneratedProtocolMessageType('RestoreRun', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _RESTORERUN_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RestoreRun.Response)
    })
  ,
  'DESCRIPTOR' : _RESTORERUN,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RestoreRun)
  })
_sym_db.RegisterMessage(RestoreRun)
_sym_db.RegisterMessage(RestoreRun.Response)

LogMetric = _reflection.GeneratedProtocolMessageType('LogMetric', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _LOGMETRIC_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogMetric.Response)
    })
  ,
  'DESCRIPTOR' : _LOGMETRIC,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.LogMetric)
  })
_sym_db.RegisterMessage(LogMetric)
_sym_db.RegisterMessage(LogMetric.Response)

LogParam = _reflection.GeneratedProtocolMessageType('LogParam', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _LOGPARAM_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogParam.Response)
    })
  ,
  'DESCRIPTOR' : _LOGPARAM,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.LogParam)
  })
_sym_db.RegisterMessage(LogParam)
_sym_db.RegisterMessage(LogParam.Response)

SetExperimentTag = _reflection.GeneratedProtocolMessageType('SetExperimentTag', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _SETEXPERIMENTTAG_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetExperimentTag.Response)
    })
  ,
  'DESCRIPTOR' : _SETEXPERIMENTTAG,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.SetExperimentTag)
  })
_sym_db.RegisterMessage(SetExperimentTag)
_sym_db.RegisterMessage(SetExperimentTag.Response)

SetTag = _reflection.GeneratedProtocolMessageType('SetTag', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _SETTAG_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetTag.Response)
    })
  ,
  'DESCRIPTOR' : _SETTAG,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.SetTag)
  })
_sym_db.RegisterMessage(SetTag)
_sym_db.RegisterMessage(SetTag.Response)

DeleteTag = _reflection.GeneratedProtocolMessageType('DeleteTag', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _DELETETAG_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteTag.Response)
    })
  ,
  'DESCRIPTOR' : _DELETETAG,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.DeleteTag)
  })
_sym_db.RegisterMessage(DeleteTag)
_sym_db.RegisterMessage(DeleteTag.Response)

GetRun = _reflection.GeneratedProtocolMessageType('GetRun', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETRUN_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetRun.Response)
    })
  ,
  'DESCRIPTOR' : _GETRUN,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetRun)
  })
_sym_db.RegisterMessage(GetRun)
_sym_db.RegisterMessage(GetRun.Response)

SearchRuns = _reflection.GeneratedProtocolMessageType('SearchRuns', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _SEARCHRUNS_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchRuns.Response)
    })
  ,
  'DESCRIPTOR' : _SEARCHRUNS,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.SearchRuns)
  })
_sym_db.RegisterMessage(SearchRuns)
_sym_db.RegisterMessage(SearchRuns.Response)

ListArtifacts = _reflection.GeneratedProtocolMessageType('ListArtifacts', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _LISTARTIFACTS_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListArtifacts.Response)
    })
  ,
  'DESCRIPTOR' : _LISTARTIFACTS,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.ListArtifacts)
  })
_sym_db.RegisterMessage(ListArtifacts)
_sym_db.RegisterMessage(ListArtifacts.Response)

FileInfo = _reflection.GeneratedProtocolMessageType('FileInfo', (_message.Message,), {
  'DESCRIPTOR' : _FILEINFO,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.FileInfo)
  })
_sym_db.RegisterMessage(FileInfo)

GetMetricHistory = _reflection.GeneratedProtocolMessageType('GetMetricHistory', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETMETRICHISTORY_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetMetricHistory.Response)
    })
  ,
  'DESCRIPTOR' : _GETMETRICHISTORY,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetMetricHistory)
  })
_sym_db.RegisterMessage(GetMetricHistory)
_sym_db.RegisterMessage(GetMetricHistory.Response)

MetricWithRunId = _reflection.GeneratedProtocolMessageType('MetricWithRunId', (_message.Message,), {
  'DESCRIPTOR' : _METRICWITHRUNID,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.MetricWithRunId)
  })
_sym_db.RegisterMessage(MetricWithRunId)

GetMetricHistoryBulkInterval = _reflection.GeneratedProtocolMessageType('GetMetricHistoryBulkInterval', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETMETRICHISTORYBULKINTERVAL_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetMetricHistoryBulkInterval.Response)
    })
  ,
  'DESCRIPTOR' : _GETMETRICHISTORYBULKINTERVAL,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetMetricHistoryBulkInterval)
  })
_sym_db.RegisterMessage(GetMetricHistoryBulkInterval)
_sym_db.RegisterMessage(GetMetricHistoryBulkInterval.Response)

LogBatch = _reflection.GeneratedProtocolMessageType('LogBatch', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _LOGBATCH_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogBatch.Response)
    })
  ,
  'DESCRIPTOR' : _LOGBATCH,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.LogBatch)
  })
_sym_db.RegisterMessage(LogBatch)
_sym_db.RegisterMessage(LogBatch.Response)

LogModel = _reflection.GeneratedProtocolMessageType('LogModel', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _LOGMODEL_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogModel.Response)
    })
  ,
  'DESCRIPTOR' : _LOGMODEL,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.LogModel)
  })
_sym_db.RegisterMessage(LogModel)
_sym_db.RegisterMessage(LogModel.Response)

LogInputs = _reflection.GeneratedProtocolMessageType('LogInputs', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _LOGINPUTS_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogInputs.Response)
    })
  ,
  'DESCRIPTOR' : _LOGINPUTS,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.LogInputs)
  })
_sym_db.RegisterMessage(LogInputs)
_sym_db.RegisterMessage(LogInputs.Response)

GetExperimentByName = _reflection.GeneratedProtocolMessageType('GetExperimentByName', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETEXPERIMENTBYNAME_RESPONSE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetExperimentByName.Response)
    })
  ,
  'DESCRIPTOR' : _GETEXPERIMENTBYNAME,
  '__module__' : 'service_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetExperimentByName)
  })
_sym_db.RegisterMessage(GetExperimentByName)
_sym_db.RegisterMessage(GetExperimentByName.Response)

_MLFLOWSERVICE = DESCRIPTOR.services_by_name['MlflowService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024org.mlflow.api.proto\220\001\001\342?\002\020\001'
  _DATASETINPUT.fields_by_name['dataset']._options = None
  _DATASETINPUT.fields_by_name['dataset']._serialized_options = b'\370\206\031\001'
  _INPUTTAG.fields_by_name['key']._options = None
  _INPUTTAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
  _INPUTTAG.fields_by_name['value']._options = None
  _INPUTTAG.fields_by_name['value']._serialized_options = b'\370\206\031\001'
  _DATASET.fields_by_name['name']._options = None
  _DATASET.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _DATASET.fields_by_name['digest']._options = None
  _DATASET.fields_by_name['digest']._serialized_options = b'\370\206\031\001'
  _DATASET.fields_by_name['source_type']._options = None
  _DATASET.fields_by_name['source_type']._serialized_options = b'\370\206\031\001'
  _DATASET.fields_by_name['source']._options = None
  _DATASET.fields_by_name['source']._serialized_options = b'\370\206\031\001'
  _CREATEEXPERIMENT.fields_by_name['name']._options = None
  _CREATEEXPERIMENT.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _CREATEEXPERIMENT._options = None
  _CREATEEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _SEARCHEXPERIMENTS._options = None
  _SEARCHEXPERIMENTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _GETEXPERIMENT.fields_by_name['experiment_id']._options = None
  _GETEXPERIMENT.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
  _GETEXPERIMENT._options = None
  _GETEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _DELETEEXPERIMENT.fields_by_name['experiment_id']._options = None
  _DELETEEXPERIMENT.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
  _DELETEEXPERIMENT._options = None
  _DELETEEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _RESTOREEXPERIMENT.fields_by_name['experiment_id']._options = None
  _RESTOREEXPERIMENT.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
  _RESTOREEXPERIMENT._options = None
  _RESTOREEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _UPDATEEXPERIMENT.fields_by_name['experiment_id']._options = None
  _UPDATEEXPERIMENT.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
  _UPDATEEXPERIMENT._options = None
  _UPDATEEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _CREATERUN._options = None
  _CREATERUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _UPDATERUN._options = None
  _UPDATERUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _DELETERUN.fields_by_name['run_id']._options = None
  _DELETERUN.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
  _DELETERUN._options = None
  _DELETERUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _RESTORERUN.fields_by_name['run_id']._options = None
  _RESTORERUN.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
  _RESTORERUN._options = None
  _RESTORERUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _LOGMETRIC.fields_by_name['key']._options = None
  _LOGMETRIC.fields_by_name['key']._serialized_options = b'\370\206\031\001'
  _LOGMETRIC.fields_by_name['value']._options = None
  _LOGMETRIC.fields_by_name['value']._serialized_options = b'\370\206\031\001'
  _LOGMETRIC.fields_by_name['timestamp']._options = None
  _LOGMETRIC.fields_by_name['timestamp']._serialized_options = b'\370\206\031\001'
  _LOGMETRIC._options = None
  _LOGMETRIC._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _LOGPARAM.fields_by_name['key']._options = None
  _LOGPARAM.fields_by_name['key']._serialized_options = b'\370\206\031\001'
  _LOGPARAM.fields_by_name['value']._options = None
  _LOGPARAM.fields_by_name['value']._serialized_options = b'\370\206\031\001'
  _LOGPARAM._options = None
  _LOGPARAM._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _SETEXPERIMENTTAG.fields_by_name['experiment_id']._options = None
  _SETEXPERIMENTTAG.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
  _SETEXPERIMENTTAG.fields_by_name['key']._options = None
  _SETEXPERIMENTTAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
  _SETEXPERIMENTTAG.fields_by_name['value']._options = None
  _SETEXPERIMENTTAG.fields_by_name['value']._serialized_options = b'\370\206\031\001'
  _SETEXPERIMENTTAG._options = None
  _SETEXPERIMENTTAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _SETTAG.fields_by_name['key']._options = None
  _SETTAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
  _SETTAG.fields_by_name['value']._options = None
  _SETTAG.fields_by_name['value']._serialized_options = b'\370\206\031\001'
  _SETTAG._options = None
  _SETTAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _DELETETAG.fields_by_name['run_id']._options = None
  _DELETETAG.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
  _DELETETAG.fields_by_name['key']._options = None
  _DELETETAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
  _DELETETAG._options = None
  _DELETETAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _GETRUN._options = None
  _GETRUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _SEARCHRUNS._options = None
  _SEARCHRUNS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _LISTARTIFACTS._options = None
  _LISTARTIFACTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _GETMETRICHISTORY.fields_by_name['metric_key']._options = None
  _GETMETRICHISTORY.fields_by_name['metric_key']._serialized_options = b'\370\206\031\001'
  _GETMETRICHISTORY._options = None
  _GETMETRICHISTORY._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _GETMETRICHISTORYBULKINTERVAL.fields_by_name['metric_key']._options = None
  _GETMETRICHISTORYBULKINTERVAL.fields_by_name['metric_key']._serialized_options = b'\370\206\031\001'
  _GETMETRICHISTORYBULKINTERVAL._options = None
  _GETMETRICHISTORYBULKINTERVAL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]\342?1\n/com.databricks.mlflow.api.MlflowTrackingMessage'
  _LOGBATCH._options = None
  _LOGBATCH._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _LOGMODEL._options = None
  _LOGMODEL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _LOGINPUTS.fields_by_name['run_id']._options = None
  _LOGINPUTS.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
  _LOGINPUTS._options = None
  _LOGINPUTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]\342?1\n/com.databricks.mlflow.api.MlflowTrackingMessage'
  _GETEXPERIMENTBYNAME.fields_by_name['experiment_name']._options = None
  _GETEXPERIMENTBYNAME.fields_by_name['experiment_name']._serialized_options = b'\370\206\031\001'
  _GETEXPERIMENTBYNAME._options = None
  _GETEXPERIMENTBYNAME._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _MLFLOWSERVICE.methods_by_name['getExperimentByName']._options = None
  _MLFLOWSERVICE.methods_by_name['getExperimentByName']._serialized_options = b'\362\206\031H\n,\n\003GET\022\037/mlflow/experiments/get-by-name\032\004\010\002\020\000\020\001*\026Get Experiment By Name'
  _MLFLOWSERVICE.methods_by_name['createExperiment']._options = None
  _MLFLOWSERVICE.methods_by_name['createExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/create\032\004\010\002\020\000\020\001*\021Create Experiment'
  _MLFLOWSERVICE.methods_by_name['searchExperiments']._options = None
  _MLFLOWSERVICE.methods_by_name['searchExperiments']._serialized_options = b'\362\206\031i\n(\n\004POST\022\032/mlflow/experiments/search\032\004\010\002\020\000\n\'\n\003GET\022\032/mlflow/experiments/search\032\004\010\002\020\000\020\001*\022Search Experiments'
  _MLFLOWSERVICE.methods_by_name['getExperiment']._options = None
  _MLFLOWSERVICE.methods_by_name['getExperiment']._serialized_options = b'\362\206\0318\n$\n\003GET\022\027/mlflow/experiments/get\032\004\010\002\020\000\020\001*\016Get Experiment\272\214\031\000'
  _MLFLOWSERVICE.methods_by_name['deleteExperiment']._options = None
  _MLFLOWSERVICE.methods_by_name['deleteExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/delete\032\004\010\002\020\000\020\001*\021Delete Experiment'
  _MLFLOWSERVICE.methods_by_name['restoreExperiment']._options = None
  _MLFLOWSERVICE.methods_by_name['restoreExperiment']._serialized_options = b'\362\206\031A\n)\n\004POST\022\033/mlflow/experiments/restore\032\004\010\002\020\000\020\001*\022Restore Experiment'
  _MLFLOWSERVICE.methods_by_name['updateExperiment']._options = None
  _MLFLOWSERVICE.methods_by_name['updateExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/update\032\004\010\002\020\000\020\001*\021Update Experiment'
  _MLFLOWSERVICE.methods_by_name['createRun']._options = None
  _MLFLOWSERVICE.methods_by_name['createRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/create\032\004\010\002\020\000\020\001*\nCreate Run'
  _MLFLOWSERVICE.methods_by_name['updateRun']._options = None
  _MLFLOWSERVICE.methods_by_name['updateRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/update\032\004\010\002\020\000\020\001*\nUpdate Run'
  _MLFLOWSERVICE.methods_by_name['deleteRun']._options = None
  _MLFLOWSERVICE.methods_by_name['deleteRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/delete\032\004\010\002\020\000\020\001*\nDelete Run'
  _MLFLOWSERVICE.methods_by_name['restoreRun']._options = None
  _MLFLOWSERVICE.methods_by_name['restoreRun']._serialized_options = b'\362\206\0313\n\"\n\004POST\022\024/mlflow/runs/restore\032\004\010\002\020\000\020\001*\013Restore Run'
  _MLFLOWSERVICE.methods_by_name['logMetric']._options = None
  _MLFLOWSERVICE.methods_by_name['logMetric']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/log-metric\032\004\010\002\020\000\020\001*\nLog Metric'
  _MLFLOWSERVICE.methods_by_name['logParam']._options = None
  _MLFLOWSERVICE.methods_by_name['logParam']._serialized_options = b'\362\206\0317\n(\n\004POST\022\032/mlflow/runs/log-parameter\032\004\010\002\020\000\020\001*\tLog Param'
  _MLFLOWSERVICE.methods_by_name['setExperimentTag']._options = None
  _MLFLOWSERVICE.methods_by_name['setExperimentTag']._serialized_options = b'\362\206\031L\n4\n\004POST\022&/mlflow/experiments/set-experiment-tag\032\004\010\002\020\000\020\001*\022Set Experiment Tag'
  _MLFLOWSERVICE.methods_by_name['setTag']._options = None
  _MLFLOWSERVICE.methods_by_name['setTag']._serialized_options = b'\362\206\031/\n\"\n\004POST\022\024/mlflow/runs/set-tag\032\004\010\002\020\000\020\001*\007Set Tag'
  _MLFLOWSERVICE.methods_by_name['deleteTag']._options = None
  _MLFLOWSERVICE.methods_by_name['deleteTag']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/delete-tag\032\004\010\002\020\000\020\001*\nDelete Tag'
  _MLFLOWSERVICE.methods_by_name['getRun']._options = None
  _MLFLOWSERVICE.methods_by_name['getRun']._serialized_options = b'\362\206\031*\n\035\n\003GET\022\020/mlflow/runs/get\032\004\010\002\020\000\020\001*\007Get Run\272\214\031\000'
  _MLFLOWSERVICE.methods_by_name['searchRuns']._options = None
  _MLFLOWSERVICE.methods_by_name['searchRuns']._serialized_options = b'\362\206\0312\n!\n\004POST\022\023/mlflow/runs/search\032\004\010\002\020\000\020\001*\013Search Runs'
  _MLFLOWSERVICE.methods_by_name['listArtifacts']._options = None
  _MLFLOWSERVICE.methods_by_name['listArtifacts']._serialized_options = b'\362\206\0317\n#\n\003GET\022\026/mlflow/artifacts/list\032\004\010\002\020\000\020\001*\016List Artifacts'
  _MLFLOWSERVICE.methods_by_name['getMetricHistory']._options = None
  _MLFLOWSERVICE.methods_by_name['getMetricHistory']._serialized_options = b'\362\206\031@\n(\n\003GET\022\033/mlflow/metrics/get-history\032\004\010\002\020\000\020\001*\022Get Metric History'
  _MLFLOWSERVICE.methods_by_name['getMetricHistoryBulkInterval']._options = None
  _MLFLOWSERVICE.methods_by_name['getMetricHistoryBulkInterval']._serialized_options = b'\362\206\031:\n6\n\003GET\022)/mlflow/metrics/get-history-bulk-interval\032\004\010\002\020\013\020\003'
  _MLFLOWSERVICE.methods_by_name['logBatch']._options = None
  _MLFLOWSERVICE.methods_by_name['logBatch']._serialized_options = b'\362\206\0313\n$\n\004POST\022\026/mlflow/runs/log-batch\032\004\010\002\020\000\020\001*\tLog Batch'
  _MLFLOWSERVICE.methods_by_name['logModel']._options = None
  _MLFLOWSERVICE.methods_by_name['logModel']._serialized_options = b'\362\206\0313\n$\n\004POST\022\026/mlflow/runs/log-model\032\004\010\002\020\000\020\001*\tLog Model'
  _MLFLOWSERVICE.methods_by_name['logInputs']._options = None
  _MLFLOWSERVICE.methods_by_name['logInputs']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/log-inputs\032\004\010\002\020\000\020\001*\nLog Inputs'
  _VIEWTYPE._serialized_start=5407
  _VIEWTYPE._serialized_end=5461
  _SOURCETYPE._serialized_start=5463
  _SOURCETYPE._serialized_end=5536
  _RUNSTATUS._serialized_start=5538
  _RUNSTATUS._serialized_end=5615
  _METRIC._serialized_start=66
  _METRIC._serialized_end=138
  _PARAM._serialized_start=140
  _PARAM._serialized_end=175
  _RUN._serialized_start=177
  _RUN._serialized_end=279
  _RUNDATA._serialized_start=281
  _RUNDATA._serialized_end=384
  _RUNINPUTS._serialized_start=386
  _RUNINPUTS._serialized_end=443
  _RUNTAG._serialized_start=445
  _RUNTAG._serialized_end=481
  _EXPERIMENTTAG._serialized_start=483
  _EXPERIMENTTAG._serialized_end=526
  _RUNINFO._serialized_start=529
  _RUNINFO._serialized_end=750
  _EXPERIMENT._serialized_start=753
  _EXPERIMENT._serialized_end=940
  _DATASETINPUT._serialized_start=942
  _DATASETINPUT._serialized_end=1028
  _INPUTTAG._serialized_start=1030
  _INPUTTAG._serialized_end=1080
  _DATASET._serialized_start=1083
  _DATASET._serialized_end=1216
  _CREATEEXPERIMENT._serialized_start=1219
  _CREATEEXPERIMENT._serialized_end=1401
  _CREATEEXPERIMENT_RESPONSE._serialized_start=1323
  _CREATEEXPERIMENT_RESPONSE._serialized_end=1356
  _SEARCHEXPERIMENTS._serialized_start=1404
  _SEARCHEXPERIMENTS._serialized_end=1658
  _SEARCHEXPERIMENTS_RESPONSE._serialized_start=1537
  _SEARCHEXPERIMENTS_RESPONSE._serialized_end=1613
  _GETEXPERIMENT._serialized_start=1661
  _GETEXPERIMENT._serialized_end=1802
  _GETEXPERIMENT_RESPONSE._serialized_start=1707
  _GETEXPERIMENT_RESPONSE._serialized_end=1757
  _DELETEEXPERIMENT._serialized_start=1804
  _DELETEEXPERIMENT._serialized_end=1908
  _DELETEEXPERIMENT_RESPONSE._serialized_start=1323
  _DELETEEXPERIMENT_RESPONSE._serialized_end=1333
  _RESTOREEXPERIMENT._serialized_start=1910
  _RESTOREEXPERIMENT._serialized_end=2015
  _RESTOREEXPERIMENT_RESPONSE._serialized_start=1323
  _RESTOREEXPERIMENT_RESPONSE._serialized_end=1333
  _UPDATEEXPERIMENT._serialized_start=2017
  _UPDATEEXPERIMENT._serialized_end=2139
  _UPDATEEXPERIMENT_RESPONSE._serialized_start=1323
  _UPDATEEXPERIMENT_RESPONSE._serialized_end=1333
  _CREATERUN._serialized_start=2142
  _CREATERUN._serialized_end=2344
  _CREATERUN_RESPONSE._serialized_start=2263
  _CREATERUN_RESPONSE._serialized_end=2299
  _UPDATERUN._serialized_start=2347
  _UPDATERUN._serialized_end=2555
  _UPDATERUN_RESPONSE._serialized_start=2465
  _UPDATERUN_RESPONSE._serialized_end=2510
  _DELETERUN._serialized_start=2557
  _DELETERUN._serialized_end=2647
  _DELETERUN_RESPONSE._serialized_start=1323
  _DELETERUN_RESPONSE._serialized_end=1333
  _RESTORERUN._serialized_start=2649
  _RESTORERUN._serialized_end=2740
  _RESTORERUN_RESPONSE._serialized_start=1323
  _RESTORERUN_RESPONSE._serialized_end=1333
  _LOGMETRIC._serialized_start=2743
  _LOGMETRIC._serialized_end=2927
  _LOGMETRIC_RESPONSE._serialized_start=1323
  _LOGMETRIC_RESPONSE._serialized_end=1333
  _LOGPARAM._serialized_start=2930
  _LOGPARAM._serialized_end=3071
  _LOGPARAM_RESPONSE._serialized_start=1323
  _LOGPARAM_RESPONSE._serialized_end=1333
  _SETEXPERIMENTTAG._serialized_start=3074
  _SETEXPERIMENTTAG._serialized_end=3218
  _SETEXPERIMENTTAG_RESPONSE._serialized_start=1323
  _SETEXPERIMENTTAG_RESPONSE._serialized_end=1333
  _SETTAG._serialized_start=3221
  _SETTAG._serialized_end=3360
  _SETTAG_RESPONSE._serialized_start=1323
  _SETTAG_RESPONSE._serialized_end=1333
  _DELETETAG._serialized_start=3362
  _DELETETAG._serialized_end=3471
  _DELETETAG_RESPONSE._serialized_start=1323
  _DELETETAG_RESPONSE._serialized_end=1333
  _GETRUN._serialized_start=3473
  _GETRUN._serialized_end=3598
  _GETRUN_RESPONSE._serialized_start=2263
  _GETRUN_RESPONSE._serialized_end=2299
  _SEARCHRUNS._serialized_start=3601
  _SEARCHRUNS._serialized_end=3881
  _SEARCHRUNS_RESPONSE._serialized_start=3774
  _SEARCHRUNS_RESPONSE._serialized_end=3836
  _LISTARTIFACTS._serialized_start=3884
  _LISTARTIFACTS._serialized_end=4100
  _LISTARTIFACTS_RESPONSE._serialized_start=3969
  _LISTARTIFACTS_RESPONSE._serialized_end=4055
  _FILEINFO._serialized_start=4102
  _FILEINFO._serialized_end=4161
  _GETMETRICHISTORY._serialized_start=4164
  _GETMETRICHISTORY._serialized_end=4398
  _GETMETRICHISTORY_RESPONSE._serialized_start=4285
  _GETMETRICHISTORY_RESPONSE._serialized_end=4353
  _METRICWITHRUNID._serialized_start=4400
  _METRICWITHRUNID._serialized_end=4497
  _GETMETRICHISTORYBULKINTERVAL._serialized_start=4500
  _GETMETRICHISTORYBULKINTERVAL._serialized_end=4783
  _GETMETRICHISTORYBULKINTERVAL_RESPONSE._serialized_start=4634
  _GETMETRICHISTORYBULKINTERVAL_RESPONSE._serialized_end=4686
  _LOGBATCH._serialized_start=4786
  _LOGBATCH._serialized_end=4963
  _LOGBATCH_RESPONSE._serialized_start=1323
  _LOGBATCH_RESPONSE._serialized_end=1333
  _LOGMODEL._serialized_start=4965
  _LOGMODEL._serialized_end=5068
  _LOGMODEL_RESPONSE._serialized_start=1323
  _LOGMODEL_RESPONSE._serialized_end=1333
  _LOGINPUTS._serialized_start=5071
  _LOGINPUTS._serialized_end=5253
  _LOGINPUTS_RESPONSE._serialized_start=1323
  _LOGINPUTS_RESPONSE._serialized_end=1333
  _GETEXPERIMENTBYNAME._serialized_start=5256
  _GETEXPERIMENTBYNAME._serialized_end=5405
  _GETEXPERIMENTBYNAME_RESPONSE._serialized_start=1707
  _GETEXPERIMENTBYNAME_RESPONSE._serialized_end=1757
  _MLFLOWSERVICE._serialized_start=5618
  _MLFLOWSERVICE._serialized_end=8872
MlflowService = service_reflection.GeneratedServiceType('MlflowService', (_service.Service,), dict(
  DESCRIPTOR = _MLFLOWSERVICE,
  __module__ = 'service_pb2'
  ))

MlflowService_Stub = service_reflection.GeneratedServiceStubType('MlflowService_Stub', (MlflowService,), dict(
  DESCRIPTOR = _MLFLOWSERVICE,
  __module__ = 'service_pb2'
  ))


# @@protoc_insertion_point(module_scope)



import contextlib
import inspect
import logging
import uuid
import warnings
from copy import deepcopy

from packaging.version import Version

import mlflow
from mlflow.entities import RunTag
from mlflow.exceptions import MlflowException
from mlflow.langchain.runnables import get_runnable_steps
from mlflow.ml_package_versions import _ML_PACKAGE_VERSIONS
from mlflow.tracking.context import registry as context_registry
from mlflow.utils.autologging_utils import (
    ExceptionSafeAbstractClass,
    disable_autologging,
    get_autologging_config,
)
from mlflow.utils.autologging_utils.safety import _resolve_extra_tags

MIN_REQ_VERSION = Version(_ML_PACKAGE_VERSIONS["langchain"]["autologging"]["minimum"])
MAX_REQ_VERSION = Version(_ML_PACKAGE_VERSIONS["langchain"]["autologging"]["maximum"])

_logger = logging.getLogger(__name__)

UNSUPPORT_LOG_MODEL_MESSAGE = (
    "MLflow autologging does not support logging models containing BaseRetriever because "
    "logging the model requires `loader_fn` and `persist_dir`. Please log the model manually "
    "using `mlflow.langchain.log_model(model, artifact_path, loader_fn=..., persist_dir=...)`"
)
INFERENCE_FILE_NAME = "inference_inputs_outputs.json"


def _get_input_data_from_function(func_name, model, args, kwargs):
    func_param_name_mapping = {
        "__call__": "inputs",
        "invoke": "input",
        "get_relevant_documents": "query",
    }
    input_example_exc = None
    if param_name := func_param_name_mapping.get(func_name):
        inference_func = getattr(model, func_name)
        # A guard to make sure `param_name` is the first argument of inference function
        if next(iter(inspect.signature(inference_func).parameters.keys())) != param_name:
            input_example_exc = MlflowException(
                "Inference function signature changes, please contact MLflow team to "
                "fix langchain autologging.",
            )
        else:
            return args[0] if len(args) > 0 else kwargs.get(param_name)
    else:
        input_example_exc = MlflowException(
            f"Unsupported inference function. Only support {list(func_param_name_mapping.keys())}."
        )
    _logger.warning(
        f"Failed to gather input example of model {model.__class__.__name__} "
        f"due to {input_example_exc}."
    )


def _convert_data_to_dict(data, key):
    if isinstance(data, dict):
        return {f"{key}-{k}": v for k, v in data.items()}
    if isinstance(data, list):
        return {key: data}
    if isinstance(data, str):
        return {key: [data]}
    raise MlflowException("Unsupported data type.")


def _combine_input_and_output(input, output, session_id, func_name):
    """
    Combine input and output into a single dictionary
    """
    if func_name == "get_relevant_documents" and output is not None:
        output = [{"page_content": doc.page_content, "metadata": doc.metadata} for doc in output]
        # to make sure output is inside a single row when converted into pandas DataFrame
        output = [output]
    result = {"session_id": [session_id]}
    if input:
        result.update(_convert_data_to_dict(input, "input"))
    if output:
        result.update(_convert_data_to_dict(output, "output"))
    return result


def _update_langchain_model_config(model):
    try:
        from langchain_core.pydantic_v1 import Extra
    except ImportError as e:
        warnings.warn(
            "MLflow langchain autologging might log model several "
            "times due to the pydantic.config.Extra import error. "
            f"Error: {e}"
        )
        return False
    else:
        # Langchain models are Pydantic models, and the value for extra is
        # ignored, we need to set it to allow so as to set attributes on
        # the model to keep track of logging status
        if hasattr(model, "__config__"):
            model.__config__.extra = Extra.allow
        return True


def _inject_mlflow_callback(func_name, mlflow_callback, args, kwargs):
    if func_name == "invoke":
        from langchain.schema.runnable.config import RunnableConfig

        in_args = False
        # `config` is the second positional argument of runnable.invoke function
        # https://github.com/langchain-ai/langchain/blob/7d444724d7582386de347fb928619c2243bd0e55/libs/core/langchain_core/runnables/base.py#L468
        if len(args) >= 2:
            config = args[1]
            in_args = True
        else:
            config = kwargs.get("config", None)
        if config is None:
            callbacks = [mlflow_callback]
            config = RunnableConfig(callbacks=callbacks)
        else:
            callbacks = config.get("callbacks") or []
            if not isinstance(callbacks, list):
                callbacks = [callbacks]
            config["callbacks"] = [*callbacks, mlflow_callback]
        if in_args:
            args = (args[0], config) + args[2:]
        else:
            kwargs["config"] = config
        return args, kwargs

    if func_name == "__call__":
        # `callbacks` is the third positional argument of chain.__call__ function
        # https://github.com/langchain-ai/langchain/blob/7d444724d7582386de347fb928619c2243bd0e55/libs/langchain/langchain/chains/base.py#L320
        if len(args) >= 3:
            callbacks = args[2] or []
            if not isinstance(callbacks, list):
                callbacks = [callbacks]
            args = args[:2] + ([*callbacks, mlflow_callback],) + args[3:]
        else:
            callbacks = kwargs.get("callbacks") or []
            if not isinstance(callbacks, list):
                callbacks = [callbacks]
            kwargs["callbacks"] = [*callbacks, mlflow_callback]
        return args, kwargs

    # https://github.com/langchain-ai/langchain/blob/7d444724d7582386de347fb928619c2243bd0e55/libs/core/langchain_core/retrievers.py#L173
    if func_name == "get_relevant_documents":
        callbacks = kwargs.get("callbacks") or []
        if not isinstance(callbacks, list):
            callbacks = [callbacks]
        kwargs["callbacks"] = [*callbacks, mlflow_callback]
        return args, kwargs


def _runnable_with_retriever(model):
    from langchain.schema import BaseRetriever

    with contextlib.suppress(ImportError):
        from langchain.schema.runnable import RunnableBranch, RunnableParallel, RunnableSequence
        from langchain.schema.runnable.passthrough import RunnableAssign

        if isinstance(model, RunnableBranch):
            return any(_runnable_with_retriever(runnable) for _, runnable in model.branches)

        if isinstance(model, RunnableParallel):
            return any(
                _runnable_with_retriever(runnable)
                for runnable in get_runnable_steps(model).values()
            )

        if isinstance(model, RunnableSequence):
            return any(_runnable_with_retriever(runnable) for runnable in get_runnable_steps(model))

        if isinstance(model, RunnableAssign):
            return _runnable_with_retriever(model.mapper)

    return isinstance(model, BaseRetriever)


def _chain_with_retriever(model):
    with contextlib.suppress(ImportError):
        from langchain.chains import RetrievalQA

        return isinstance(model, RetrievalQA)
    return False


def patched_inference(func_name, original, self, *args, **kwargs):
    """
    A patched implementation of langchain models inference process which enables logging the
    following parameters, metrics and artifacts:

    - model
    - metrics
    - data

    We patch either `invoke` or `__call__` function for different models
    based on their usage.
    """

    import langchain
    from langchain_community.callbacks import MlflowCallbackHandler

    class _MlflowLangchainCallback(MlflowCallbackHandler, metaclass=ExceptionSafeAbstractClass):
        """
        Callback for auto-logging metrics and parameters.
        We need to inherit ExceptionSafeAbstractClass to avoid invalid new
        input arguments added to original function call.
        """

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)

    _lc_version = Version(langchain.__version__)
    if not MIN_REQ_VERSION <= _lc_version <= MAX_REQ_VERSION:
        warnings.warn(
            "Autologging is known to be compatible with langchain versions between "
            f"{MIN_REQ_VERSION} and {MAX_REQ_VERSION} and may not succeed with packages "
            "outside this range."
        )

    run_id = getattr(self, "run_id", None)
    active_run = mlflow.active_run()
    if run_id is None:
        # only log the tags once
        extra_tags = get_autologging_config(mlflow.langchain.FLAVOR_NAME, "extra_tags", None)
        # include run context tags
        resolved_tags = context_registry.resolve_tags(extra_tags)
        tags = _resolve_extra_tags(mlflow.langchain.FLAVOR_NAME, resolved_tags)
        if active_run:
            run_id = active_run.info.run_id
            mlflow.MlflowClient().log_batch(
                run_id=run_id,
                tags=[RunTag(key, str(value)) for key, value in tags.items()],
            )
    else:
        tags = None
    # Use session_id-inference_id as artifact directory where mlflow
    # callback logs artifacts into, to avoid overriding artifacts
    session_id = getattr(self, "session_id", uuid.uuid4().hex)
    inference_id = getattr(self, "inference_id", 0)
    mlflow_callback = _MlflowLangchainCallback(
        tracking_uri=mlflow.get_tracking_uri(),
        run_id=run_id,
        artifacts_dir=f"artifacts-{session_id}-{inference_id}",
        tags=tags,
    )
    args, kwargs = _inject_mlflow_callback(func_name, mlflow_callback, args, kwargs)
    with disable_autologging():
        result = original(self, *args, **kwargs)

    mlflow_callback.flush_tracker()

    log_models = get_autologging_config(mlflow.langchain.FLAVOR_NAME, "log_models", False)
    log_input_examples = get_autologging_config(
        mlflow.langchain.FLAVOR_NAME, "log_input_examples", False
    )
    log_model_signatures = get_autologging_config(
        mlflow.langchain.FLAVOR_NAME, "log_model_signatures", False
    )
    input_example = None
    if log_models and not hasattr(self, "model_logged"):
        if (
            (func_name == "get_relevant_documents")
            or _runnable_with_retriever(self)
            or _chain_with_retriever(self)
        ):
            _logger.info(UNSUPPORT_LOG_MODEL_MESSAGE)
        else:
            # warn user in case we did't capture some cases where retriever is used
            warnings.warn(UNSUPPORT_LOG_MODEL_MESSAGE)
            if log_input_examples:
                input_example = deepcopy(
                    _get_input_data_from_function(func_name, self, args, kwargs)
                )
                if not log_model_signatures:
                    _logger.info(
                        "Signature is automatically generated for logged model if "
                        "input_example is provided. To disable log_model_signatures, "
                        "please also disable log_input_examples."
                    )

            registered_model_name = get_autologging_config(
                mlflow.langchain.FLAVOR_NAME, "registered_model_name", None
            )
            try:
                with disable_autologging():
                    mlflow.langchain.log_model(
                        self,
                        "model",
                        input_example=input_example,
                        registered_model_name=registered_model_name,
                        run_id=mlflow_callback.mlflg.run_id,
                    )
            except Exception as e:
                _logger.warning(f"Failed to log model due to error {e}.")
            if _update_langchain_model_config(self):
                self.model_logged = True

    # Even if the model is not logged, we keep a single run per model
    if _update_langchain_model_config(self):
        if not hasattr(self, "run_id"):
            self.run_id = mlflow_callback.mlflg.run_id
        if not hasattr(self, "session_id"):
            self.session_id = session_id
        self.inference_id = inference_id + 1

    log_inputs_outputs = get_autologging_config(
        mlflow.langchain.FLAVOR_NAME, "log_inputs_outputs", False
    )
    if log_inputs_outputs:
        if input_example is None:
            input_data = deepcopy(_get_input_data_from_function(func_name, self, args, kwargs))
            if input_data is None:
                _logger.info("Input data gathering failed, only log inference results.")
        else:
            input_data = input_example
        try:
            data_dict = _combine_input_and_output(input_data, result, self.session_id, func_name)
        except Exception as e:
            _logger.warning(
                f"Failed to log inputs and outputs into `{INFERENCE_FILE_NAME}` "
                f"file due to error {e}."
            )
        mlflow.log_table(data_dict, INFERENCE_FILE_NAME, run_id=mlflow_callback.mlflg.run_id)

    # Terminate the run if it is not managed by the user
    if active_run is None or active_run.info.run_id != mlflow_callback.mlflg.run_id:
        mlflow.MlflowClient().set_terminated(mlflow_callback.mlflg.run_id)

    return result



import logging
from abc import ABCMeta, abstractmethod
from time import sleep, time

from mlflow.entities.model_registry import ModelVersionTag
from mlflow.entities.model_registry.model_version_status import ModelVersionStatus
from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS, ErrorCode
from mlflow.utils.annotations import developer_stable

_logger = logging.getLogger(__name__)

AWAIT_MODEL_VERSION_CREATE_SLEEP_INTERVAL_SECONDS = 3


@developer_stable
class AbstractStore:
    """
    Abstract class that defines API interfaces for storing Model Registry metadata.
    """

    __metaclass__ = ABCMeta

    def __init__(self, store_uri=None, tracking_uri=None):
        """
        Empty constructor. This is deliberately not marked as abstract, else every derived class
        would be forced to create one.

        Args:
            store_uri: The model registry store URI.
            tracking_uri: URI of the current MLflow tracking server, used to perform operations
                like fetching source run metadata or downloading source run artifacts
                to support subsequently uploading them to the model registry storage
                location.
        """
        pass

    # CRUD API for RegisteredModel objects

    @abstractmethod
    def create_registered_model(self, name, tags=None, description=None):
        """
        Create a new registered model in backend store.

        Args:
            name: Name of the new model. This is expected to be unique in the backend store.
            tags: A list of :py:class:`mlflow.entities.model_registry.RegisteredModelTag`
                instances associated with this registered model.
            description: Description of the model.

        Returns:
            A single object of :py:class:`mlflow.entities.model_registry.RegisteredModel`
            created in the backend.

        """
        pass

    @abstractmethod
    def update_registered_model(self, name, description):
        """
        Update description of the registered model.

        Args:
            name: Registered model name.
            description: New description.

        Returns:
            A single updated :py:class:`mlflow.entities.model_registry.RegisteredModel` object.
        """
        pass

    @abstractmethod
    def rename_registered_model(self, name, new_name):
        """
        Rename the registered model.

        Args:
            name: Registered model name.
            new_name: New proposed name.

        Returns:
            A single updated :py:class:`mlflow.entities.model_registry.RegisteredModel` object.
        """
        pass

    @abstractmethod
    def delete_registered_model(self, name):
        """
        Delete the registered model.
        Backend raises exception if a registered model with given name does not exist.

        Args:
            name: Registered model name.

        Returns:
            None
        """
        pass

    @abstractmethod
    def search_registered_models(
        self, filter_string=None, max_results=None, order_by=None, page_token=None
    ):
        """
        Search for registered models in backend that satisfy the filter criteria.

        Args:
            filter_string: Filter query string, defaults to searching all registered models.
            max_results: Maximum number of registered models desired.
            order_by: List of column names with ASC|DESC annotation, to be used for ordering
                matching search results.
            page_token: Token specifying the next page of results. It should be obtained from
                a ``search_registered_models`` call.

        Returns:
            A PagedList of :py:class:`mlflow.entities.model_registry.RegisteredModel` objects
            that satisfy the search expressions. The pagination token for the next page can be
            obtained via the ``token`` attribute of the object.
        """
        pass

    @abstractmethod
    def get_registered_model(self, name):
        """
        Get registered model instance by name.

        Args:
            name: Registered model name.

        Returns:
            A single :py:class:`mlflow.entities.model_registry.RegisteredModel` object.
        """
        pass

    @abstractmethod
    def get_latest_versions(self, name, stages=None):
        """
        Latest version models for each requested stage. If no ``stages`` argument is provided,
        returns the latest version for each stage.

        Args:
            name: Registered model name.
            stages: List of desired stages. If input list is None, return latest versions for
                each stage.

        Returns:
            List of :py:class:`mlflow.entities.model_registry.ModelVersion` objects.
        """
        pass

    @abstractmethod
    def set_registered_model_tag(self, name, tag):
        """
        Set a tag for the registered model.

        Args:
            name: Registered model name.
            tag: :py:class:`mlflow.entities.model_registry.RegisteredModelTag` instance to log.

        Returns:
            None
        """
        pass

    @abstractmethod
    def delete_registered_model_tag(self, name, key):
        """
        Delete a tag associated with the registered model.

        Args:
            name: Registered model name.
            key: Registered model tag key.

        Returns:
            None
        """
        pass

    # CRUD API for ModelVersion objects

    @abstractmethod
    def create_model_version(
        self,
        name,
        source,
        run_id=None,
        tags=None,
        run_link=None,
        description=None,
        local_model_path=None,
    ):
        """
        Create a new model version from given source and run ID.

        Args:
            name: Registered model name.
            source: URI indicating the location of the model artifacts.
            run_id: Run ID from MLflow tracking server that generated the model.
            tags: A list of :py:class:`mlflow.entities.model_registry.ModelVersionTag`
                instances associated with this model version.
            run_link: Link to the run from an MLflow tracking server that generated this model.
            description: Description of the version.
            local_model_path: Local path to the MLflow model, if it's already accessible
                on the local filesystem. Can be used by AbstractStores that
                upload model version files to the model registry to avoid
                a redundant download from the source location when logging
                and registering a model via a single
                mlflow.<flavor>.log_model(..., registered_model_name) call

        Returns:
            A single object of :py:class:`mlflow.entities.model_registry.ModelVersion`
            created in the backend.

        """
        pass

    @abstractmethod
    def update_model_version(self, name, version, description):
        """
        Update metadata associated with a model version in backend.

        Args:
            name: Registered model name.
            version: Registered model version.
            description: New model description.

        Returns:
            A single :py:class:`mlflow.entities.model_registry.ModelVersion` object.
        """
        pass

    @abstractmethod
    def transition_model_version_stage(self, name, version, stage, archive_existing_versions):
        """
        Update model version stage.

        Args:
            name: Registered model name.
            version: Registered model version.
            stage: New desired stage for this model version.
            archive_existing_versions: If this flag is set to ``True``, all existing model
                versions in the stage will be automatically moved to the "archived" stage. Only
                valid when ``stage`` is ``"staging"`` or ``"production"`` otherwise an error will
                be raised.

        Returns:
            A single :py:class:`mlflow.entities.model_registry.ModelVersion` object.

        """
        pass

    @abstractmethod
    def delete_model_version(self, name, version):
        """
        Delete model model version in backend.

        Args:
            name: Registered model name.
            version: Registered model version.

        Returns:
            None
        """
        pass

    @abstractmethod
    def get_model_version(self, name, version):
        """
        Get the model version instance by name and version.

        Args:
            name: Registered model name.
            version: Registered model version.

        Returns:
            A single :py:class:`mlflow.entities.model_registry.ModelVersion` object.
        """
        pass

    @abstractmethod
    def get_model_version_download_uri(self, name, version):
        """
        Get the download location in Model Registry for this model version.
        NOTE: For first version of Model Registry, since the models are not copied over to another
              location, download URI points to input source path.

        Args:
            name: Registered model name.
            version: Registered model version.

        Returns:
            A single URI location that allows reads for downloading.
        """
        pass

    @abstractmethod
    def search_model_versions(
        self, filter_string=None, max_results=None, order_by=None, page_token=None
    ):
        """
        Search for model versions in backend that satisfy the filter criteria.

        Args:
            filter_string: A filter string expression. Currently supports a single filter
                condition either name of model like ``name = 'model_name'`` or
                ``run_id = '...'``.
            max_results: Maximum number of model versions desired.
            order_by: List of column names with ASC|DESC annotation, to be used for ordering
                matching search results.
            page_token: Token specifying the next page of results. It should be obtained from
                a ``search_model_versions`` call.

        Returns:
            A PagedList of :py:class:`mlflow.entities.model_registry.ModelVersion`
            objects that satisfy the search expressions. The pagination token for the next
            page can be obtained via the ``token`` attribute of the object.

        """
        pass

    @abstractmethod
    def set_model_version_tag(self, name, version, tag):
        """
        Set a tag for the model version.

        Args:
            name: Registered model name.
            version: Registered model version.
            tag: :py:class:`mlflow.entities.model_registry.ModelVersionTag` instance to log.

        Returns:
            None
        """
        pass

    @abstractmethod
    def delete_model_version_tag(self, name, version, key):
        """
        Delete a tag associated with the model version.

        Args:
            name: Registered model name.
            version: Registered model version.
            key: Tag key.

        Returns:
            None
        """
        pass

    @abstractmethod
    def set_registered_model_alias(self, name, alias, version):
        """
        Set a registered model alias pointing to a model version.

        Args:
            name: Registered model name.
            alias: Name of the alias.
            version: Registered model version number.

        Returns:
            None
        """
        pass

    @abstractmethod
    def delete_registered_model_alias(self, name, alias):
        """
        Delete an alias associated with a registered model.

        Args:
            name: Registered model name.
            alias: Name of the alias.

        Returns:
            None
        """
        pass

    @abstractmethod
    def get_model_version_by_alias(self, name, alias):
        """
        Get the model version instance by name and alias.

        Args:
            name: Registered model name.
            alias: Name of the alias.

        Returns:
            A single :py:class:`mlflow.entities.model_registry.ModelVersion` object.
        """
        pass

    def copy_model_version(self, src_mv, dst_name):
        """
        Copy a model version from one registered model to another as a new model version.

        Args:
            src_mv: A :py:class:`mlflow.entities.model_registry.ModelVersion` object representing
                the source model version.
            dst_name: The name of the registered model to copy the model version to. If a
                registered model with this name does not exist, it will be created.

        Returns:
            Single :py:class:`mlflow.entities.model_registry.ModelVersion` object representing
            the cloned model version.
        """
        try:
            self.create_registered_model(dst_name)
        except MlflowException as e:
            if e.error_code != ErrorCode.Name(RESOURCE_ALREADY_EXISTS):
                raise

        try:
            mv_copy = self.create_model_version(
                name=dst_name,
                source=f"models:/{src_mv.name}/{src_mv.version}",
                run_id=src_mv.run_id,
                tags=[ModelVersionTag(k, v) for k, v in src_mv.tags.items()],
                run_link=src_mv.run_link,
                description=src_mv.description,
            )
        except MlflowException as e:
            raise MlflowException(
                f"Failed to create model version copy. The current model registry backend "
                f"may not yet support model version URI sources.\nError: {e}"
            ) from e

        return mv_copy

    def _await_model_version_creation(self, mv, await_creation_for):
        """
        Await for model version to become ready after creation.

        Args:
            mv: A :py:class:`mlflow.entities.model_registry.ModelVersion` object.
            await_creation_for: Number of seconds to wait for the model version to finish being
                created and is in ``READY`` status.
        """
        self._await_model_version_creation_impl(mv, await_creation_for)

    def _await_model_version_creation_impl(self, mv, await_creation_for, hint=""):
        _logger.info(
            f"Waiting up to {await_creation_for} seconds for model version to finish creation. "
            f"Model name: {mv.name}, version {mv.version}",
        )
        max_time = time() + await_creation_for
        pending_status = ModelVersionStatus.to_string(ModelVersionStatus.PENDING_REGISTRATION)
        while mv.status == pending_status:
            if time() > max_time:
                raise MlflowException(
                    f"Exceeded max wait time for model name: {mv.name} version: {mv.version} "
                    f"to become READY. Status: {mv.status} Wait Time: {await_creation_for}"
                    f".{hint}"
                )
            mv = self.get_model_version(mv.name, mv.version)
            sleep(AWAIT_MODEL_VERSION_CREATE_SLEEP_INTERVAL_SECONDS)
        if mv.status != ModelVersionStatus.to_string(ModelVersionStatus.READY):
            raise MlflowException(
                f"Model version creation failed for model name: {mv.name} version: "
                f"{mv.version} with status: {mv.status} and message: {mv.status_message}"
            )



import textwrap
import warnings
from functools import wraps
from typing import Dict

import importlib_metadata
from packaging.version import Version

from mlflow.ml_package_versions import _ML_PACKAGE_VERSIONS
from mlflow.utils.autologging_utils.versioning import (
    FLAVOR_TO_MODULE_NAME_AND_VERSION_INFO_KEY,
)


def _create_placeholder(key: str):
    return "{{ " + key + " }}"


def _replace_keys_with_placeholders(d: Dict) -> Dict:
    return {_create_placeholder(k): v for k, v in d.items()}


def _get_indentation_of_key(line: str, placeholder: str) -> str:
    index = line.find(placeholder)
    return (index * " ") if index != -1 else ""


def _indent(text: str, indent: str) -> str:
    """Indent everything but first line in text."""
    lines = text.splitlines()
    if len(lines) <= 1:
        return text

    else:
        first_line = lines[0]
        subsequent_lines = "\n".join(list(lines[1:]))
        indented_subsequent_lines = textwrap.indent(subsequent_lines, indent)
        return first_line + "\n" + indented_subsequent_lines


def _replace_all(text: str, replacements: Dict[str, str]) -> str:
    """
    Replace all instances of replacements.keys() with their corresponding
    values in text. The replacements will be inserted on the same line
    with wrapping to the same level of indentation, for example:

    ```
    Args:
        param_1: {{ key }}
    ```

    will become...

    ```
    Args:
        param_1: replaced_value_at same indentation as prior
                 and if there are more lines they will also
                 have the same indentation.
    ```
    """
    for key, value in replacements.items():
        if key in text:
            indent = _get_indentation_of_key(text, key)
            indented_value = _indent(value, indent)
            text = text.replace(key, indented_value)
    return text


class ParamDocs(dict):
    """
    Represents a set of parameter documents in the docstring.
    """

    def __repr__(self):
        return f"ParamDocs({super().__repr__()})"

    def format(self, **kwargs):
        """
        Formats values to be substituted in via the format_docstring() method.

        Args:
            kwargs: A `dict` in the form of `{"< placeholder name >": "< value >"}`.

        Returns:
            A new `ParamDocs` instance with the formatted param docs.

        .. code-block:: text
            :caption: Example

            >>> pd = ParamDocs(p1="{{ doc1 }}", p2="{{ doc2 }}")
            >>> pd.format(doc1="foo", doc2="bar")
            ParamDocs({'p1': 'foo', 'p2': 'bar'})
        """
        replacements = _replace_keys_with_placeholders(kwargs)
        return ParamDocs({k: _replace_all(v, replacements) for k, v in self.items()})

    def format_docstring(self, docstring: str) -> str:
        """
        Formats placeholders in `docstring`.

        Args:
            p1: {{ p1 }}
            p2: {{ p2 }}

        .. code-block:: text
            :caption: Example

            >>> pd = ParamDocs(p1="doc1", p2="doc2
            doc2 second line")
            >>> docstring = '''
            ... Args:
            ...     p1: {{ p1 }}
            ...     p2: {{ p2 }}
            ... '''.strip()
            >>> print(pd.format_docstring(docstring))
        """
        if docstring is None:
            return None

        replacements = _replace_keys_with_placeholders(self)
        lines = docstring.splitlines()
        for i, line in enumerate(lines):
            lines[i] = _replace_all(line, replacements)

        return "\n".join(lines)


def format_docstring(param_docs):
    """
    Returns a decorator that replaces param doc placeholders (e.g. '{{ param_name }}') in the
    docstring of the decorated function.

    Args:
        param_docs: A `ParamDocs` instance or `dict`.

    Returns:
        A decorator to apply the formatting.

    .. code-block:: text
        :caption: Example

        >>> param_docs = {"p1": "doc1", "p2": "doc2
        doc2 second line"}
        >>> @format_docstring(param_docs)
        ... def func(p1, p2):
        ...     '''
        ...     Args:
        ...         p1: {{ p1 }}
        ...         p2: {{ p2 }}
        ...     '''
        >>> import textwrap
        >>> print(textwrap.dedent(func.__doc__).strip())

        Args:
            p1: doc1
            p2: doc2
                doc2 second line
    """
    param_docs = ParamDocs(param_docs)

    def decorator(func):
        func.__doc__ = param_docs.format_docstring(func.__doc__)
        return func

    return decorator


# `{{ ... }}` represents a placeholder.
LOG_MODEL_PARAM_DOCS = ParamDocs(
    {
        "conda_env": (
            """Either a dictionary representation of a Conda environment or the path to a conda
environment yaml file. If provided, this describes the environment this model should be run in.
At a minimum, it should specify the dependencies contained in :func:`get_default_conda_env()`.
If ``None``, a conda environment with pip requirements inferred by
:func:`mlflow.models.infer_pip_requirements` is added
to the model. If the requirement inference fails, it falls back to using
:func:`get_default_pip_requirements`. pip requirements from ``conda_env`` are written to a pip
``requirements.txt`` file and the full conda environment is written to ``conda.yaml``.
The following is an *example* dictionary representation of a conda environment::

    {
        "name": "mlflow-env",
        "channels": ["conda-forge"],
        "dependencies": [
            "python=3.8.15",
            {
                "pip": [
                    "{{ package_name }}==x.y.z"
                ],
            },
        ],
    }"""
        ),
        "pip_requirements": (
            """Either an iterable of pip requirement strings
(e.g. ``["{{ package_name }}", "-r requirements.txt", "-c constraints.txt"]``) or the string path to
a pip requirements file on the local filesystem (e.g. ``"requirements.txt"``). If provided, this
describes the environment this model should be run in. If ``None``, a default list of requirements
is inferred by :func:`mlflow.models.infer_pip_requirements` from the current software environment.
If the requirement inference fails, it falls back to using :func:`get_default_pip_requirements`.
Both requirements and constraints are automatically parsed and written to ``requirements.txt`` and
``constraints.txt`` files, respectively, and stored as part of the model. Requirements are also
written to the ``pip`` section of the model's conda environment (``conda.yaml``) file."""
        ),
        "extra_pip_requirements": (
            """Either an iterable of pip
requirement strings
(e.g. ``["pandas", "-r requirements.txt", "-c constraints.txt"]``) or the string path to
a pip requirements file on the local filesystem (e.g. ``"requirements.txt"``). If provided, this
describes additional pip requirements that are appended to a default set of pip requirements
generated automatically based on the user's current software environment. Both requirements and
constraints are automatically parsed and written to ``requirements.txt`` and ``constraints.txt``
files, respectively, and stored as part of the model. Requirements are also written to the ``pip``
section of the model's conda environment (``conda.yaml``) file.

.. warning::
    The following arguments can't be specified at the same time:

    - ``conda_env``
    - ``pip_requirements``
    - ``extra_pip_requirements``

`This example <https://github.com/mlflow/mlflow/blob/master/examples/pip_requirements/pip_requirements.py>`_ demonstrates how to specify pip requirements using
``pip_requirements`` and ``extra_pip_requirements``."""  # noqa: E501
        ),
        "signature": (
            """an instance of the :py:class:`ModelSignature <mlflow.models.ModelSignature>`
class that describes the model's inputs and outputs. If not specified but an
``input_example`` is supplied, a signature will be automatically inferred
based on the supplied input example and model. To disable automatic signature
inference when providing an input example, set ``signature`` to ``False``.
To manually infer a model signature, call
:py:func:`infer_signature() <mlflow.models.infer_signature>` on datasets
with valid model inputs, such as a training dataset with the target column
omitted, and valid model outputs, like model predictions made on the training
dataset, for example:

.. code-block:: python

    from mlflow.models import infer_signature

    train = df.drop_column("target_label")
    predictions = ...  # compute model predictions
    signature = infer_signature(train, predictions)
"""
        ),
        "input_example": (
            """one or several instances of valid model input. The input example is used
as a hint of what data to feed the model. It will be converted to a Pandas
DataFrame and then serialized to json using the Pandas split-oriented
format, or a numpy array where the example will be serialized to json
by converting it to a list. Bytes are base64-encoded. When the ``signature`` parameter is
``None``, the input example is used to infer a model signature.
"""
        ),
        "example_no_conversion": (
            """If ``True``, the input example will not be converted to a Pandas DataFrame
format when saving. This is useful when the model expects a non-DataFrame input and the
input example could be passed directly to the model. Defaults to ``False`` for backwards
compatibility.
"""
        ),
        "prompt_template": (
            """A string that, if provided, will be used to format the user's input prior
to inference. The string should contain a single placeholder, ``{prompt}``, which will be
replaced with the user's input. For example: ``"Answer the following question. Q: {prompt} A:"``.

Currently, only the following pipeline types are supported:

- `feature-extraction <https://huggingface.co/transformers/main_classes/pipelines.html#transformers.FeatureExtractionPipeline>`_
- `fill-mask <https://huggingface.co/transformers/main_classes/pipelines.html#transformers.FillMaskPipeline>`_
- `summarization <https://huggingface.co/transformers/main_classes/pipelines.html#transformers.SummarizationPipeline>`_
- `text2text-generation <https://huggingface.co/transformers/main_classes/pipelines.html#transformers.Text2TextGenerationPipeline>`_
- `text-generation <https://huggingface.co/transformers/main_classes/pipelines.html#transformers.TextGenerationPipeline>`_
"""
        ),
        "code_paths": (
            """A list of local filesystem paths to Python file dependencies (or directories
containing file dependencies). These files are *prepended* to the system path when the model
is loaded. Files declared as dependencies for a given model should have relative
imports declared from a common root path if multiple files are defined with import dependencies
between them to avoid import errors when loading the model.

For a detailed explanation of ``code_paths`` functionality, recommended usage patterns and
limitations, see the
`code_paths usage guide <https://mlflow.org/docs/latest/model/dependencies.html?highlight=code_paths#saving-extra-code-with-an-mlflow-model>`_.
"""
        ),
        "save_pretrained": (
            """If set to ``False``, MLflow will not save the Transformer model weight files,
instead only saving the reference to the HuggingFace Hub model repository and its commit hash.
This is useful when you load the pretrained model from HuggingFace Hub and want to log or save
it to MLflow without modifying the model weights. In such case, specifying this flag to
``False`` will save the storage space and reduce time to save the model. Please refer to the
:ref:`Storage-Efficient Model Logging <transformers-save-pretrained-guide>` for more detailed usage.


.. warning::

    If the model is saved with ``save_pretrained`` set to ``False``, the model cannot be
    registered to the MLflow Model Registry. In order to convert the model to the one that
    can be registered, you can use :py:func:`mlflow.transformers.persist_pretrained_model()`
    to download the model weights from the HuggingFace Hub and save it in the existing model
    artifacts. Please refer to :ref:`Transformers flavor documentation <persist-pretrained-guide>`
    for more detailed usage.

    .. code-block:: python

        import mlflow.transformers

        model_uri = "YOUR_MODEL_URI_LOGGED_WITH_SAVE_PRETRAINED_FALSE"
        model = mlflow.transformers.persist_pretrained_model(model_uri)
        mlflow.register_model(model_uri, "model_name")

.. important::

    When you save the `PEFT <https://huggingface.co/docs/peft/en/index>`_ model, MLflow will
    override the `save_pretrained` flag to `False` and only store the PEFT adapter weights. The
    base model weights are not saved but the reference to the HuggingFace repository and
    its commit hash are logged instead.
"""
        ),
    }
)


def get_module_min_and_max_supported_ranges(module_name):
    """
    Extracts the minimum and maximum supported package versions from the provided module name.
    The version information is provided via the yaml-to-python-script generation script in
    dev/update_ml_package_versions.py which writes a python file to the importable namespace of
    mlflow.ml_package_versions

    Args:
        module_name: The string name of the module as it is registered in ml_package_versions.py

    Returns:
        tuple of minimum supported version, maximum supported version as strings.
    """
    versions = _ML_PACKAGE_VERSIONS[module_name]["models"]
    min_version = versions["minimum"]
    max_version = versions["maximum"]
    return min_version, max_version


def docstring_version_compatibility_warning(integration_name):
    """
    Generates a docstring that can be applied as a note stating a version compatibility range for
    a given flavor.

    Args:
        integration_name: The name of the module as stored within ml-package-versions.yml

    Returns:
        The wrapped function with the additional docstring header applied
    """

    def annotated_func(func):
        # NB: if using this decorator, ensure the package name to module name reference is
        # updated with the flavor's `save` and `load` functions being used within the dictionary
        # mlflow.utils.autologging_utils.versioning.FLAVOR_TO_MODULE_NAME_AND_VERSION_INFO_KEY
        _, module_key = FLAVOR_TO_MODULE_NAME_AND_VERSION_INFO_KEY[integration_name]
        min_ver, max_ver = get_module_min_and_max_supported_ranges(module_key)
        required_pkg_versions = f"``{min_ver}`` -  ``{max_ver}``"

        notice = (
            f"The '{integration_name}' MLflow Models integration is known to be compatible with "
            f"the following package version ranges: {required_pkg_versions}. "
            f"MLflow Models integrations with {integration_name} may not succeed when used with "
            "package versions outside of this range."
        )

        @wraps(func)
        def version_func(*args, **kwargs):
            installed_version = Version(importlib_metadata.version(module_key))
            if installed_version < Version(min_ver) or installed_version > Version(max_ver):
                warnings.warn(notice, category=FutureWarning, stacklevel=2)
            return func(*args, **kwargs)

        version_func.__doc__ = (
            "    .. Note:: " + notice + "\n" * 2 + func.__doc__ if func.__doc__ else notice
        )

        return version_func

    return annotated_func



import os
from typing import Any, Dict, Optional, Union

import yaml

import mlflow

__mlflow_model_config__ = None


# TODO: Let ModelConfig take in a dictionary instead of a file path
class ModelConfig:
    """
    ModelConfig used in code to read a YAML configuration file, and this configuration file can be
    overridden when logging a model.
    """

    def __init__(self, *, development_config: Optional[Union[str, Dict[str, Any]]] = None):
        config = globals().get("__mlflow_model_config__", None)
        # backwards compatibility
        rag_config = getattr(mlflow.langchain._rag_utils, "__databricks_rag_config_path__", None)

        # Here mlflow_model_config have 3 states:
        # 1. None, this means if the mlflow_model_config is None, use development_config if
        # available
        # 2. "", Empty string, this means the users explicitly didn't set the model config
        # while logging the model so if ModelConfig is used, it should throw an error
        # 3. A valid path, this means the users have set the model config while logging the
        # model so use that path
        if config is not None:
            self.config = config
        elif rag_config is not None:
            self.config = rag_config
        else:
            self.config = development_config

        if not self.config:
            raise FileNotFoundError(
                "Config file is not provided which is needed to load the model. "
                "Please provide a valid path."
            )

        if not isinstance(self.config, dict) and not os.path.isfile(self.config):
            raise FileNotFoundError(f"Config file '{self.config}' not found.")

    def _read_config(self):
        """Reads the YAML configuration file and returns its contents.

        Raises:
            FileNotFoundError: If the configuration file does not exist.
            yaml.YAMLError: If there is an error parsing the YAML content.

        Returns:
            dict or None: The content of the YAML file as a dictionary, or None if the
            config path is not set.
        """
        if isinstance(self.config, dict):
            return self.config

        with open(self.config) as file:
            try:
                return yaml.safe_load(file)
            except yaml.YAMLError as e:
                raise yaml.YAMLError(f"Error parsing YAML file: {e}")

    def get(self, key):
        """Gets the value of a top-level parameter in the configuration."""
        config_data = self._read_config()

        if config_data and key in config_data:
            return config_data[key]
        else:
            raise KeyError(f"Key '{key}' not found in configuration: {config_data}.")


def _set_model_config(model_config):
    globals()["__mlflow_model_config__"] = model_config



"""
NOTE: The contents of this file have been inlined from the wrapt package's source code
https://github.com/GrahamDumpleton/wrapt/blob/1.12.1/src/wrapt/importer.py.
Some modifications, have been made in order to:
    - avoid duplicate registration of import hooks
    - inline functions from dependent wrapt submodules rather than importing them.

This module implements a post import hook mechanism styled after what is described in PEP-369.
Note that it doesn't cope with modules being reloaded.
It also extends the functionality to support custom hooks for import errors
(as opposed to only successful imports).
"""

import importlib  # noqa: F401
import sys
import threading

string_types = (str,)


# from .decorators import synchronized
# NOTE: Instead of using this import (from wrapt's decorator module, see
# https://github.com/GrahamDumpleton/wrapt/blob/68316bea668fd905a4acb21f37f12596d8c30d80/src/wrapt/decorators.py#L430-L456),
# we define a decorator with similar behavior that acquires a lock while calling the decorated
# function
def synchronized(lock):
    def decorator(f):
        # See e.g. https://www.python.org/dev/peps/pep-0318/#examples
        def new_fn(*args, **kwargs):
            with lock:
                return f(*args, **kwargs)

        return new_fn

    return decorator


# The dictionary registering any post import hooks to be triggered once
# the target module has been imported. Once a module has been imported
# and the hooks fired, the list of hooks recorded against the target
# module will be truncated but the list left in the dictionary. This
# acts as a flag to indicate that the module had already been imported.

_post_import_hooks = {}
_post_import_hooks_lock = threading.RLock()

# A dictionary for any import hook error handlers to be triggered when the
# target module import fails.

_import_error_hooks = {}
_import_error_hooks_lock = threading.RLock()

_import_hook_finder_init = False

# Register a new post import hook for the target module name. This
# differs from the PEP-369 implementation in that it also allows the
# hook function to be specified as a string consisting of the name of
# the callback in the form 'module:function'. This will result in a
# proxy callback being registered which will defer loading of the
# specified module containing the callback function until required.


def _create_import_hook_from_string(name):
    def import_hook(module):
        module_name, function = name.split(":")
        attrs = function.split(".")
        __import__(module_name)
        callback = sys.modules[module_name]
        for attr in attrs:
            callback = getattr(callback, attr)
        return callback(module)

    return import_hook


def register_generic_import_hook(hook, name, hook_dict, overwrite):
    # Create a deferred import hook if hook is a string name rather than
    # a callable function.

    if isinstance(hook, string_types):
        hook = _create_import_hook_from_string(hook)

    # Automatically install the import hook finder if it has not already
    # been installed.

    global _import_hook_finder_init
    if not _import_hook_finder_init:
        _import_hook_finder_init = True
        sys.meta_path.insert(0, ImportHookFinder())

    # Determine if any prior registration of an import hook for
    # the target modules has occurred and act appropriately.

    hooks = hook_dict.get(name, None)

    if hooks is None:
        # No prior registration of import hooks for the target
        # module. We need to check whether the module has already been
        # imported. If it has we fire the hook immediately and add an
        # empty list to the registry to indicate that the module has
        # already been imported and hooks have fired. Otherwise add
        # the post import hook to the registry.

        module = sys.modules.get(name, None)

        if module is not None:
            hook_dict[name] = []
            hook(module)

        else:
            hook_dict[name] = [hook]

    elif hooks == []:
        # A prior registration of import hooks for the target
        # module was done and the hooks already fired. Fire the hook
        # immediately.

        module = sys.modules[name]
        hook(module)

    else:
        # A prior registration of import hooks for the target
        # module was done but the module has not yet been imported.

        def hooks_equal(existing_hook, hook):
            if hasattr(existing_hook, "__name__") and hasattr(hook, "__name__"):
                return existing_hook.__name__ == hook.__name__
            else:
                return False

        if overwrite:
            hook_dict[name] = [
                existing_hook
                for existing_hook in hook_dict[name]
                if not hooks_equal(existing_hook, hook)
            ]

        hook_dict[name].append(hook)


@synchronized(_import_error_hooks_lock)
def register_import_error_hook(hook, name, overwrite=True):
    """
    Args:
        hook: A function or string entrypoint to invoke when the specified module is imported
            and an error occurs.
        name: The name of the module for which to fire the hook at import error detection time.
        overwrite: Specifies the desired behavior when a preexisting hook for the same
            function / entrypoint already exists for the specified module. If `True`,
            all preexisting hooks matching the specified function / entrypoint will be
            removed and replaced with a single instance of the specified `hook`.
    """
    register_generic_import_hook(hook, name, _import_error_hooks, overwrite)


@synchronized(_post_import_hooks_lock)
def register_post_import_hook(hook, name, overwrite=True):
    """
    Args:
        hook: A function or string entrypoint to invoke when the specified module is imported.
        name: The name of the module for which to fire the hook at import time.
        overwrite: Specifies the desired behavior when a preexisting hook for the same
            function / entrypoint already exists for the specified module. If `True`,
            all preexisting hooks matching the specified function / entrypoint will be
            removed and replaced with a single instance of the specified `hook`.
    """
    register_generic_import_hook(hook, name, _post_import_hooks, overwrite)


# Register post import hooks defined as package entry points.


def _create_import_hook_from_entrypoint(entrypoint):
    def import_hook(module):
        __import__(entrypoint.module_name)
        callback = sys.modules[entrypoint.module_name]
        for attr in entrypoint.attrs:
            callback = getattr(callback, attr)
        return callback(module)

    return import_hook


def discover_post_import_hooks(group):
    # New in 3.9: https://docs.python.org/3/library/importlib.resources.html#importlib.resources.files
    if sys.version_info.major > 2 and sys.version_info.minor > 8:
        from importlib.resources import files  # clint: disable=lazy-builtin-import

        for entrypoint in (
            resource.name for resource in files(group).iterdir() if resource.is_file()
        ):
            callback = _create_import_hook_from_entrypoint(entrypoint)
            register_post_import_hook(callback, entrypoint.name)
    else:
        from importlib.resources import contents  # clint: disable=lazy-builtin-import

        for entrypoint in contents(group):
            callback = _create_import_hook_from_entrypoint(entrypoint)
            register_post_import_hook(callback, entrypoint.name)


# Indicate that a module has been loaded. Any post import hooks which
# were registered against the target module will be invoked. If an
# exception is raised in any of the post import hooks, that will cause
# the import of the target module to fail.


@synchronized(_post_import_hooks_lock)
def notify_module_loaded(module):
    name = getattr(module, "__name__", None)
    hooks = _post_import_hooks.get(name)

    if hooks:
        _post_import_hooks[name] = []

        for hook in hooks:
            hook(module)


@synchronized(_import_error_hooks_lock)
def notify_module_import_error(module_name):
    hooks = _import_error_hooks.get(module_name)

    if hooks:
        # Error hooks differ from post import hooks, in that we don't clear the
        # hook as soon as it fires.
        for hook in hooks:
            hook(module_name)


# A custom module import finder. This intercepts attempts to import
# modules and watches out for attempts to import target modules of
# interest. When a module of interest is imported, then any post import
# hooks which are registered will be invoked.


class _ImportHookChainedLoader:
    def __init__(self, loader):
        self.loader = loader

    def load_module(self, fullname):
        try:
            module = self.loader.load_module(fullname)
            notify_module_loaded(module)
        except (ImportError, AttributeError):
            notify_module_import_error(fullname)
            raise

        return module


class ImportHookFinder:
    def __init__(self):
        self.in_progress = {}

    @synchronized(_post_import_hooks_lock)
    @synchronized(_import_error_hooks_lock)
    def find_module(self, fullname, path=None):
        # If the module being imported is not one we have registered
        # import hooks for, we can return immediately. We will
        # take no further part in the importing of this module.

        if fullname not in _post_import_hooks and fullname not in _import_error_hooks:
            return None

        # When we are interested in a specific module, we will call back
        # into the import system a second time to defer to the import
        # finder that is supposed to handle the importing of the module.
        # We set an in progress flag for the target module so that on
        # the second time through we don't trigger another call back
        # into the import system and cause a infinite loop.

        if fullname in self.in_progress:
            return None

        self.in_progress[fullname] = True

        # Now call back into the import system again.

        try:
            # For Python 3 we need to use find_spec().loader
            # from the importlib.util module. It doesn't actually
            # import the target module and only finds the
            # loader. If a loader is found, we need to return
            # our own loader which will then in turn call the
            # real loader to import the module and invoke the
            # post import hooks.
            try:
                import importlib.util  # clint: disable=lazy-builtin-import

                loader = importlib.util.find_spec(fullname).loader
            # If an ImportError (or AttributeError) is encountered while finding the module,
            # notify the hooks for import errors
            except (ImportError, AttributeError):
                notify_module_import_error(fullname)
                loader = importlib.find_loader(fullname, path)
            if loader:
                return _ImportHookChainedLoader(loader)
        finally:
            del self.in_progress[fullname]

    @synchronized(_post_import_hooks_lock)
    @synchronized(_import_error_hooks_lock)
    def find_spec(self, fullname, path, target=None):
        # If the module being imported is not one we have registered
        # import hooks for, we can return immediately. We will
        # take no further part in the importing of this module.

        if fullname not in _post_import_hooks and fullname not in _import_error_hooks:
            return None

        # When we are interested in a specific module, we will call back
        # into the import system a second time to defer to the import
        # finder that is supposed to handle the importing of the module.
        # We set an in progress flag for the target module so that on
        # the second time through we don't trigger another call back
        # into the import system and cause a infinite loop.

        if fullname in self.in_progress:
            return None

        self.in_progress[fullname] = True

        # Now call back into the import system again.

        try:
            import importlib.util  # clint: disable=lazy-builtin-import

            spec = importlib.util.find_spec(fullname)
            # Replace the module spec's loader with a wrapped version that executes import
            # hooks when the module is loaded
            spec.loader = _ImportHookChainedLoader(spec.loader)
            return spec
        except (ImportError, AttributeError):
            notify_module_import_error(fullname)
        finally:
            del self.in_progress[fullname]


# Decorator for marking that a function should be called as a post
# import hook when the target module is imported.
# If error_handler is True, then apply the marked function as an import hook
# for import errors (instead of successful imports).
# It is assumed that all error hooks are added during driver start-up,
# and thus added prior to any import calls. If an error hook is added
# after a module has already failed the import, there's no guarantee
# that the hook will fire.


def when_imported(name, error_handler=False):
    def register(hook):
        if error_handler:
            register_import_error_hook(hook, name)
        else:
            register_post_import_hook(hook, name)
        return hook

    return register



from mlflow.entities._mlflow_object import _MlflowObject
from mlflow.entities.experiment_tag import ExperimentTag
from mlflow.protos.service_pb2 import Experiment as ProtoExperiment
from mlflow.protos.service_pb2 import ExperimentTag as ProtoExperimentTag


class Experiment(_MlflowObject):
    """
    Experiment object.
    """

    DEFAULT_EXPERIMENT_NAME = "Default"

    def __init__(
        self,
        experiment_id,
        name,
        artifact_location,
        lifecycle_stage,
        tags=None,
        creation_time=None,
        last_update_time=None,
    ):
        super().__init__()
        self._experiment_id = experiment_id
        self._name = name
        self._artifact_location = artifact_location
        self._lifecycle_stage = lifecycle_stage
        self._tags = {tag.key: tag.value for tag in (tags or [])}
        self._creation_time = creation_time
        self._last_update_time = last_update_time

    @property
    def experiment_id(self):
        """String ID of the experiment."""
        return self._experiment_id

    @property
    def name(self):
        """String name of the experiment."""
        return self._name

    def _set_name(self, new_name):
        self._name = new_name

    @property
    def artifact_location(self):
        """String corresponding to the root artifact URI for the experiment."""
        return self._artifact_location

    @property
    def lifecycle_stage(self):
        """Lifecycle stage of the experiment. Can either be 'active' or 'deleted'."""
        return self._lifecycle_stage

    @property
    def tags(self):
        """Tags that have been set on the experiment."""
        return self._tags

    def _add_tag(self, tag):
        self._tags[tag.key] = tag.value

    @property
    def creation_time(self):
        return self._creation_time

    def _set_creation_time(self, creation_time):
        self._creation_time = creation_time

    @property
    def last_update_time(self):
        return self._last_update_time

    def _set_last_update_time(self, last_update_time):
        self._last_update_time = last_update_time

    @classmethod
    def from_proto(cls, proto):
        experiment = cls(
            proto.experiment_id,
            proto.name,
            proto.artifact_location,
            proto.lifecycle_stage,
            # `creation_time` and `last_update_time` were added in MLflow 1.29.0. Experiments
            # created before this version don't have these fields and `proto.creation_time` and
            # `proto.last_update_time` default to 0. We should only set `creation_time` and
            # `last_update_time` if they are non-zero.
            creation_time=proto.creation_time or None,
            last_update_time=proto.last_update_time or None,
        )
        for proto_tag in proto.tags:
            experiment._add_tag(ExperimentTag.from_proto(proto_tag))
        return experiment

    def to_proto(self):
        experiment = ProtoExperiment()
        experiment.experiment_id = self.experiment_id
        experiment.name = self.name
        experiment.artifact_location = self.artifact_location
        experiment.lifecycle_stage = self.lifecycle_stage
        if self.creation_time:
            experiment.creation_time = self.creation_time
        if self.last_update_time:
            experiment.last_update_time = self.last_update_time
        experiment.tags.extend(
            [ProtoExperimentTag(key=key, value=val) for key, val in self._tags.items()]
        )
        return experiment



import re
import warnings
from pathlib import Path
from typing import Any, Dict, TypeVar
from urllib.parse import urlparse

from mlflow.artifacts import download_artifacts
from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
from mlflow.store.artifact.artifact_repository_registry import get_registered_artifact_repositories
from mlflow.utils.uri import is_local_uri


def register_artifact_dataset_sources():
    from mlflow.data.dataset_source_registry import register_dataset_source

    registered_source_schemes = set()
    artifact_schemes_to_exclude = [
        "http",
        "https",
        "runs",
        "models",
        "mlflow-artifacts",
        # DBFS supports two access patterns: dbfs:/ (URI) and /dbfs (FUSE).
        # The DBFS artifact repository online supports dbfs:/ (URI). To ensure
        # a consistent dictionary representation of DBFS datasets across the URI and
        # FUSE representations, we exclude dbfs from the set of dataset sources
        # that are autogenerated using artifact repositories and instead define
        # a separate DBFSDatasetSource elsewhere
        "dbfs",
    ]
    schemes_to_artifact_repos = get_registered_artifact_repositories()
    for scheme, artifact_repo in schemes_to_artifact_repos.items():
        if scheme in artifact_schemes_to_exclude or scheme in registered_source_schemes:
            continue

        if "ArtifactRepository" in artifact_repo.__name__:
            # Artifact repository name is something like "LocalArtifactRepository",
            # "S3ArtifactRepository", etc. To preserve capitalization, strip ArtifactRepository
            # and replace it with ArtifactDatasetSource
            dataset_source_name = artifact_repo.__name__.replace(
                "ArtifactRepository", "ArtifactDatasetSource"
            )
        else:
            # Artifact repository name has some other form, e.g. "dbfs_artifact_repo_factory".
            # In this case, generate the name by capitalizing the first letter of the scheme and
            # appending ArtifactRepository
            scheme = str(scheme)

            def camelcase_scheme(scheme):
                parts = re.split(r"[-_]", scheme)
                return "".join([part.capitalize() for part in parts])

            source_name_prefix = camelcase_scheme(scheme)
            dataset_source_name = source_name_prefix + "ArtifactDatasetSource"

        try:
            registered_source_schemes.add(scheme)
            dataset_source = _create_dataset_source_for_artifact_repo(
                scheme=scheme, dataset_source_name=dataset_source_name
            )
            register_dataset_source(dataset_source)
        except Exception as e:
            warnings.warn(
                f"Failed to register a dataset source for URIs with scheme '{scheme}': {e}",
                stacklevel=2,
            )


def _create_dataset_source_for_artifact_repo(scheme: str, dataset_source_name: str):
    from mlflow.data.filesystem_dataset_source import FileSystemDatasetSource

    if scheme in ["", "file"]:
        source_type = "local"
        class_docstring = "Represents the source of a dataset stored on the local filesystem."
    else:
        source_type = scheme
        class_docstring = (
            f"Represents a filesystem-based or blob-storage-based dataset source identified by a"
            f" URI with scheme '{scheme}'."
        )

    DatasetForArtifactRepoSourceType = TypeVar(dataset_source_name)

    class ArtifactRepoSource(FileSystemDatasetSource):
        def __init__(self, uri: str):
            self._uri = uri

        @property
        def uri(self):
            """
            The URI with scheme '{scheme}' referring to the dataset source filesystem location.

            Returns
                The URI with scheme '{scheme}' referring to the dataset source filesystem
                location.
            """
            return self._uri

        @staticmethod
        def _get_source_type() -> str:
            return source_type

        def load(self, dst_path=None) -> str:
            """
            Downloads the dataset source to the local filesystem.

            Args:
                dst_path: Path of the local filesystem destination directory to which to download
                    the dataset source. If the directory does not exist, it is created. If
                    unspecified, the dataset source is downloaded to a new uniquely-named
                    directory on the local filesystem, unless the dataset source already
                    exists on the local filesystem, in which case its local path is
                    returned directly.

            Returns:
                The path to the downloaded dataset source on the local filesystem.
            """
            return download_artifacts(artifact_uri=self.uri, dst_path=dst_path)

        @staticmethod
        def _can_resolve(raw_source: Any):
            is_local_source_type = ArtifactRepoSource._get_source_type() == "local"

            if not isinstance(raw_source, str) and (
                not isinstance(raw_source, Path) and is_local_source_type
            ):
                return False

            try:
                if is_local_source_type:
                    return is_local_uri(str(raw_source), is_tracking_or_registry_uri=False)
                else:
                    parsed_source = urlparse(str(raw_source))
                    return parsed_source.scheme == scheme
            except Exception:
                return False

        @classmethod
        def _resolve(cls, raw_source: Any) -> DatasetForArtifactRepoSourceType:
            return cls(str(raw_source))

        def to_dict(self) -> Dict[Any, Any]:
            """
            Returns:
                A JSON-compatible dictionary representation of the {dataset_source_name}.
            """
            return {
                "uri": self.uri,
            }

        @classmethod
        def from_dict(cls, source_dict: Dict[Any, Any]) -> DatasetForArtifactRepoSourceType:
            uri = source_dict.get("uri")
            if uri is None:
                raise MlflowException(
                    f'Failed to parse {dataset_source_name}. Missing expected key: "uri"',
                    INVALID_PARAMETER_VALUE,
                )

            return cls(uri=uri)

    ArtifactRepoSource.__name__ = dataset_source_name
    ArtifactRepoSource.__qualname__ = dataset_source_name
    ArtifactRepoSource.__doc__ = class_docstring
    ArtifactRepoSource.to_dict.__doc__ = ArtifactRepoSource.to_dict.__doc__.format(
        dataset_source_name=dataset_source_name
    )
    ArtifactRepoSource.uri.__doc__ = ArtifactRepoSource.uri.__doc__.format(scheme=scheme)
    return ArtifactRepoSource



# MLflow Keras 3 flavor.
import keras
from packaging.version import Version

if Version(keras.__version__) < Version("3.0.0"):
    from mlflow.tensorflow import (
        # Redirect `mlflow.keras._load_pyfunc` to `mlflow.tensorflow._load_pyfunc`,
        # For backwards compatibility on loading keras model saved by old mlflow versions.
        _load_pyfunc,
        autolog,
        load_model,
        log_model,
        save_model,
    )

    __all__ = [
        "_load_pyfunc",
        "autolog",
        "load_model",
        "save_model",
        "log_model",
    ]
else:
    from mlflow.keras.autologging import autolog
    from mlflow.keras.callback import MlflowCallback
    from mlflow.keras.load import _load_pyfunc, load_model
    from mlflow.keras.save import (
        get_default_conda_env,
        get_default_pip_requirements,
        log_model,
        save_model,
    )

    FLAVOR_NAME = "keras"

    MLflowCallback = MlflowCallback  # for backwards compatibility

    __all__ = [
        "_load_pyfunc",
        "MlflowCallback",
        "MLflowCallback",
        "autolog",
        "load_model",
        "save_model",
        "log_model",
        "get_default_pip_requirements",
        "get_default_conda_env",
    ]
